/////////////////////////////////////
//                                 //
//         PanEngine API           //
//                                 //
//         Version 2020            //
//                                 //
//        CompuTherm, LLC          //
//      Copyright, 2012-2020       //
//                                 //
//       www.computherm.com        //
//                                 //
/////////////////////////////////////

//==================================//
//                                  //
//    PanEngine Test Example 4      //
//                                  //
//    Solidification Simulation     //
//       for Sinlge Alloy           //
//       (Al-Mg-Zn System)          //
//                                  //
//==================================//


#include "std.h"
#include "PanEngineX.h"
#include "solidification.h"

#ifdef WIN32
#include <windows.h>
#endif
#include <sstream>

 bool progress_4(char* msg, Solidification_Parameter& sp, vector<Solidification_Node>& node) {
	 cout << "\nT = " << node.rbegin()->pt.m_st->m_T << " K  ";
	 for (auto ph = node.rbegin()->pt.m_ppt.begin(); ph != node.rbegin()->pt.m_ppt.end(); ++ph) {
		 if (ph != node.rbegin()->pt.m_ppt.begin()) cout << "+";
		 cout << (*ph)->m_phase_name;
	 }
	 cout << endl;
	 node.rbegin()->pt.print_debug(cout);
	 P_Point* pt = &node.rbegin()->pt;

	 for (auto ppt = pt->m_ppt.begin();  ppt != pt->m_ppt.end(); ++ppt) {
	//	 if (_stricmp((*ppt)->m_phase_name.c_str(), "Liquid") == 0) continue;
		 cout << (*ppt)->m_phase_name << " + Liquid" << endl;
		 for (auto c = pt->m_st->m_comp.begin(); c != pt->m_st->m_comp.end(); ++c) {
			 auto cmp = (*ppt)->m_st->m_comp.find((*c).first);
			 if (cmp != (*ppt)->m_st->m_comp.end()) {
				 cout << "  dxdT_S(" << (*cmp).first << ") = " << (*cmp).second->m_dxdT_S << "  ";
				 cout << "  dxdT_L(" << (*cmp).first << ") = " << (*cmp).second->m_dxdT_L << "  ";
				 cout << "  m_dxdT(" << (*cmp).first << ") = " << (*cmp).second->m_dxdT << "  ";
				 cout << endl;
			 }
		 }
	 }
	 return true;
}

string output_result(string file_name, vector<Solidification_Node>& solidification_result)
{
   ofstream output(file_name.c_str());
   output << "number of points = " << solidification_result.size() << endl;

   for (int i = 0; i < (int)solidification_result.size(); ++i )
   {
      output << "Node " << i << endl;

      output << endl;
      solidification_result[i].print_debug(output);
      output << endl;
   }

   output.close();

   return "";
}

void set_solidification_simulation_condition_4(Pan_Calculation& calc, pair<string, string> db, int alloy_number, bool is_scheil_model) {

	calc.m_name="Solidification Simulation";
	calc.m_calc_type = P_CALC_SOLIDIFICATION;

	// databases
	shared_ptr<Pan_Database> d_b = make_shared<Pan_Database>();
	d_b->m_file_name = db.first;
	d_b->m_type = db.second;
	calc.m_databases.push_back(pair<string, shared_ptr<Pan_Database>>(d_b->m_file_name, d_b));

	// units
	shared_ptr<Pan_Unit> unit_T = make_shared<Pan_Unit>();
	unit_T->m_name = "T";
	unit_T->m_unit = P_T_K;
	calc.m_units.insert(pair<string, shared_ptr<Pan_Unit>> (unit_T->m_name, unit_T));

	shared_ptr<Pan_Unit> unit_P = make_shared<Pan_Unit>();
	unit_P->m_name = "P";
	unit_P->m_unit = P_P_atm;
	calc.m_units.insert(pair<string, shared_ptr<Pan_Unit>> (unit_P->m_name, unit_P));

	// set unit for component amount
	shared_ptr<Pan_Unit> unit_m = make_shared<Pan_Unit>();
	unit_m->m_name = "n";
	//unit_m->m_unit = P_M_kg;
	unit_m->m_unit = P_M_mole;
	calc.m_units.insert(pair<string, shared_ptr<Pan_Unit>> (unit_m->m_name, unit_m));

	// system
	calc.m_system.push_back(make_shared<Pan_System>());
	shared_ptr<Pan_System> sys_ptr = calc.m_system[0];

	// components
	vector <pair<string, double>> comp_name_amount;

	switch (alloy_number) {
	case 1 :
		comp_name_amount.push_back(pair<string, double>("Al", 70));
		comp_name_amount.push_back(pair<string, double>("Mg", 3));
		comp_name_amount.push_back(pair<string, double>("Zn", 27));
		break;
	case 2 :
		comp_name_amount.push_back(pair<string, double>("Al", 70));
		comp_name_amount.push_back(pair<string, double>("Mg", 10));
		comp_name_amount.push_back(pair<string, double>("Zn", 20));
		break;
	case 3 :
		comp_name_amount.push_back(pair<string, double>("Al", 90));
		comp_name_amount.push_back(pair<string, double>("Mg", 5));
		comp_name_amount.push_back(pair<string, double>("Zn", 5));
		break;
	case 4 :
		comp_name_amount.push_back(pair<string, double>("Al", 10));
		comp_name_amount.push_back(pair<string, double>("Mg", 80));
		comp_name_amount.push_back(pair<string, double>("Zn", 10));
		break;
	default:
		comp_name_amount.push_back(pair<string, double>("Al", 20));
		comp_name_amount.push_back(pair<string, double>("Mg", 50));
		comp_name_amount.push_back(pair<string, double>("Zn", 30));
		break;
	}

	for (int i=0; i<(int)comp_name_amount.size(); ++i) {
		shared_ptr<Pan_Component> comp = make_shared<Pan_Component>();
		comp->m_name = comp_name_amount[i].first;
		comp->m_status = P_SELECTED;
		sys_ptr->m_components.push_back(comp);
	}

	// phases
	shared_ptr<Pan_Phase> phase0 = make_shared<Pan_Phase>();
	phase0->m_name = "*";
	phase0->m_status = P_PHASE_SUSPENDED;
	sys_ptr->m_phases.push_back(phase0);

	shared_ptr<Pan_Phase> phase1 = make_shared<Pan_Phase>();
	phase1->m_name = "*";
	phase1->m_status = P_PHASE_ENTERED;
	sys_ptr->m_phases.push_back(phase1);

	// starting point
	shared_ptr<Pan_Point> pt1 = make_shared<Pan_Point>();
	shared_ptr<Pan_Statespace> st;
	st = make_shared<Pan_Statespace>();
	st->m_T = 750+273;
	st->m_P = 1;

	for (int i=0; i<(int)comp_name_amount.size(); ++i) {
		st->m_n.insert(pair<string, double> (comp_name_amount[i].first, comp_name_amount[i].second));
	}

	pt1->m_statespace = st;

	sys_ptr->m_points.push_back(pt1);

	if (is_scheil_model) 
	{
		sys_ptr->m_condition->m_model = "scheil";
	}
	else 
	{
		sys_ptr->m_condition->m_model = "lever";
	}

	// calculation condition	
	sys_ptr->m_condition->m_physical_property = true;
	sys_ptr->m_condition->m_startFromLiquidusSurface = false;
	sys_ptr->m_condition->m_endWhenNoMoreLiquid = false;

	sys_ptr->m_condition->m_equilibriumType = "global";
	sys_ptr->m_condition->m_step_T_max = 20;
	sys_ptr->m_condition->m_step_T_min = 0.1;
	sys_ptr->m_condition->m_T_end = 300;
	sys_ptr->m_condition->m_liquid_amount_min = 0.005;
	sys_ptr->m_condition->m_liquid_lamda_min = 0.8;
	sys_ptr->m_condition->m_x_min = 1e-10;
}

string set_solidification_parameters_4(const Pan_Calculation& calc, Solidification_Parameter& s_param) {
	s_param.reset();

	shared_ptr<Pan_System> sys_ptr = calc.m_system[0];
	string model=sys_ptr->m_condition->m_model;
	if (_stricmp(model.c_str(), "Scheil") == 0) {
		s_param.model_type = 0;
	} else
	if (_stricmp(model.c_str(), "lever") == 0) {
		s_param.model_type = 1;
	} else {
		s_param.model_type = -1;
	}

	s_param.m_solvent_name = string("Al");
	s_param.dT_min = sys_ptr->m_condition->m_step_T_min;
	s_param.dT_max = sys_ptr->m_condition->m_step_T_max;
	s_param.T_start = sys_ptr->m_points[0]->m_statespace->m_T;	// user given starting temperature
	s_param.T_end = sys_ptr->m_condition->m_T_end;
	s_param.T_range = fabs(s_param.T_start - s_param.T_end);
	if (s_param.T_range < 1) s_param.T_range = 1;

	s_param.dT_max = sys_ptr->m_condition->m_step_T_max;
	s_param.dT_min = sys_ptr->m_condition->m_step_T_min;

	s_param.min_liq_amount = sys_ptr->m_condition->m_liquid_amount_min;	// minimum liquidus amount to stop
	s_param.min_x = sys_ptr->m_condition->m_x_min;	// minimum amount of xj
	s_param.useAverageCompositionInSolidForScheil = true;

	s_param.startFromLiquidusSurface = false;
	s_param.endWhenNoMoreLiquid = false;	
	
	//s_param.startFromLiquidusSurface = true;
	//s_param.endWhenNoMoreLiquid = true;

	// set callback function
	s_param.progress_ptr = &progress_4;
	return "";
}

int PanEngineX_Test_4()
{
	cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n";
	cout << "|  Test Example 4 : Solidification Simulation  |\n";
	cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n";

	int alloy_number = 4;
	bool is_Scheil_model = true;

	//================================
	// Define a PanEngine user pointer
	//================================
	PanEngine* user;
	char msg[256];
	string s;

	cout << "Defining user pointer:" << endl;
	user = definePanEngineUser(msg);

	if (user == NULL)
	{
		cout << "Error in defineUser" << endl;
		return 0;
	}


	//============================
	// Set PanEngine Configuration
	//============================
	// set "case_sensitive" before importing database
	cout << "Set PanEngine configuration:" << endl;
	map<string, string> config;
	config["case_sensitive"] = "true";
	s = user->pe_set_configuration(config);

	//==========================================
	// Load Thermodynamic Database in tdb Format
	//==========================================
	cout << "Reading database file:" << endl;

	pair<string, string> db = make_pair<string, string>(string("AlMgZn.tdb"), string("tdb"));

	s = user->pe_import_database(db);

	if (!s.empty())
	{
		cout << "Error in open database [" << s << "]\n";
		return 0;
	}

	//==========================================
	// Input Solidification Simulation Condition
	//==========================================
	cout << "Input solidification simulation condition:" << endl;
	Pan_Calculation calc;
	Solidification_Parameter s_param;

	set_solidification_simulation_condition_4(calc, db, alloy_number, is_Scheil_model);
	set_solidification_parameters_4(calc, s_param);

	//======================================
	// Set Simulation Condition in PanEngine
	//======================================
	cout << "Set calculation condition:" << endl;
	s = user->pe_set_calculation_condition(calc);

	if (!s.empty()) {
		cout << "Error in set calculation condition [" << s << "]\n";
		return 0;
	}

	//==========================
	// Solidification Simulating
	//==========================
	cout << "Solidification Simulating ..." << endl;

	vector <Solidification_Node> solidification_result;	// store solidification results

	string result=user->pe_solidification_simulation(s_param, calc, solidification_result);

	//output_result("my_test_4.dat", solidification_result);

	cout << "\n\n========= End of Example 4 =========\n\n";


	//==============================
	// Delete PanEngine user pointer
	//==============================
	deletePanEngineUser(user);

	return 1;
}
