/////////////////////////////////////
//                                 //
//         PanEngine API           //
//                                 //
//         Version 2020            //
//        CompuTherm, LLC          //
//       Copyright, 2012-2020      //
//                                 //
//        www.computherm.com       //
//                                 //
/////////////////////////////////////

#pragma once

#include "stl.h"

struct XMLNode;

const int max_num_chars = 30;

enum PAN_STATUS {
	P_SELECTED,
	P_SUSPENDED,
	P_STATUS_NOT_DEFINED
};


const char PAN_STATUS_NAME[][max_num_chars] = {
	"Selected",
	"Suspended",
	"Status not defined"
};

enum PAN_CALC_TYPE {	// calculation type
	P_CALC_POINT,
	P_CALC_LINE,
	P_CALC_SECTION,
	P_CALC_PROJECTION,
	P_CALC_SOLIDIFICATION,
	P_CALC_PAN_SOLIDIFICATION,
	P_CALC_DIFFUSION,
	P_CALC_PRECIPITATION,
	P_CALC_SOLUTION_TREATMENT,
	P_CALC_GRAIN_GROWTH,
	P_CALC_MICROSTRUCTURE,
	P_CALC_TTT,
	P_CALC_CCT,
	P_CALC_IPB,
	P_CALC_EBC,
	P_CALC_BAYES_TRAINING,
	P_CALC_BAYES_PREDICTION,
	P_CALC_NOT_DEFINED
};

const char PAN_CALC_TYPE_NAME[][max_num_chars] = {	// calculation type
	"point",
	"line",
	"section",
	"projection",
	"solidification",
	"pan_solidification",
	"diffusion",
	"precipitation",
	"solution_treatment",
	"grain_growth",
	"microstructure",
	"TTT",
	"CCT",
	"IPB",
	"EBC",
	"Bayes_Training",
	"Bayes_Prediction",
	"calc_type_not_defined"
};


//////////////  map <string, PAN_STATUS>  //////////////////////
static vector<pair<string, enum PAN_STATUS>> pan_create_pan_status_vector()
{
 	vector<pair<string, enum PAN_STATUS>> v;

	int i;

	int size = sizeof(PAN_STATUS_NAME)/(max_num_chars * sizeof(char));

	for (i=0; i<size; i++)
	{
		string var_name = PAN_STATUS_NAME[i];
		transform(var_name.begin(), var_name.end(), var_name.begin(), (int(*)(int)) (int(*)(int)) toupper);
		v.push_back(make_pair(var_name, (PAN_STATUS)i));
	}

	return v;
}

static map<string, enum PAN_STATUS> pan_create_pan_status_name_pan_status_map()
{
 	map<string, enum PAN_STATUS> m;

	vector<pair<string, enum PAN_STATUS>> v = pan_create_pan_status_vector();
	for (auto it=v.begin(); it!=v.end(); it++)
	{
		m[(*it).first] = (*it).second;
	}
	return m;
}

#ifdef __GNUC__ 
	/// /// __attribute__ ((visibility ("hidden"))) 
#endif
static map<string, enum PAN_STATUS> map_pan_status_name_pan_status = pan_create_pan_status_name_pan_status_map();

static enum PAN_STATUS pan_get_pan_status(string& var_name)
{
	string s=var_name;
	transform(s.begin(),s.end(),s.begin(),(int(*)(int)) (int(*)(int)) toupper);
	auto it = map_pan_status_name_pan_status.find(s);
	if (it==map_pan_status_name_pan_status.end()) return P_STATUS_NOT_DEFINED;
	return (*it).second;
}

//////////////  map <string, PAN_CALC_TYPE>  //////////////////////
static vector<pair<string, enum PAN_CALC_TYPE>> pan_create_pan_calc_type_vector()
{
 	vector<pair<string, enum PAN_CALC_TYPE>> v;

	int i;

	int size = sizeof(PAN_CALC_TYPE_NAME)/(max_num_chars * sizeof(char));

	for (i=0; i<size; i++)
	{
		string var_name = PAN_CALC_TYPE_NAME[i];
		transform(var_name.begin(), var_name.end(), var_name.begin(), (int(*)(int)) (int(*)(int)) toupper);
		v.push_back(make_pair(var_name, (PAN_CALC_TYPE)i));
	}

	return v;
}

static map<string, enum PAN_CALC_TYPE> pan_create_pan_calc_type_name_pan_calc_type_map()
{
 	map<string, enum PAN_CALC_TYPE> m;

	vector<pair<string, enum PAN_CALC_TYPE>> v = pan_create_pan_calc_type_vector();
	for (auto it=v.begin(); it!=v.end(); it++)
	{
		m[(*it).first] = (*it).second;
	}
	return m;
}

#ifdef __GNUC__ 
	/// /// __attribute__ ((visibility ("hidden"))) 
#endif
static map<string, enum PAN_CALC_TYPE> map_pan_calc_type_name_pan_calc_type = pan_create_pan_calc_type_name_pan_calc_type_map();

static enum PAN_CALC_TYPE pan_get_pan_calc_type(string& var_name)
{
	string s=var_name;
	transform(s.begin(),s.end(),s.begin(),(int(*)(int)) (int(*)(int)) toupper);
	auto it = map_pan_calc_type_name_pan_calc_type.find(s);
	if (it==map_pan_calc_type_name_pan_calc_type.end()) return P_CALC_NOT_DEFINED;
	return (*it).second;
}

////////////////////////////////////////

enum PAN_PHASE_STATUS {
	P_PHASE_ENTERED,
	P_PHASE_SUSPENDED,
	P_PHASE_DORMANT,
	P_PHASE_FIXED,
	P_PHASE_EQUILIBRIUM,
	P_PHASE_STATUS_NOT_DEFINED
};

const char PAN_PHASE_STATUS_NAME[][max_num_chars] = {
	"Entered",
	"Suspended",
	"Dormant",
	"Fixed",
	"Equilibrium",
	"Not_defined"
};


//////////////  map <string, PAN_PHASE_STATUS>  //////////////////////
static vector<pair<string, enum PAN_PHASE_STATUS>> pan_create_pan_phase_status_vector()
{
 	vector<pair<string, enum PAN_PHASE_STATUS>> v;

	int i;

	int size = sizeof(PAN_PHASE_STATUS_NAME)/(max_num_chars * sizeof(char));

	for (i=0; i<size; i++)
	{
		string var_name = PAN_PHASE_STATUS_NAME[i];
		transform(var_name.begin(), var_name.end(), var_name.begin(), (int(*)(int)) (int(*)(int)) toupper);
		v.push_back(make_pair(var_name, (PAN_PHASE_STATUS)i));
	}

	return v;
}

static map<string, enum PAN_PHASE_STATUS> pan_create_pan_phase_status_name_pan_phase_status_map()
{
 	map<string, enum PAN_PHASE_STATUS> m;

	vector<pair<string, enum PAN_PHASE_STATUS>> v = pan_create_pan_phase_status_vector();
	for (auto it=v.begin(); it!=v.end(); it++)
	{
		m[(*it).first] = (*it).second;
	}
	return m;
}

#ifdef __GNUC__ 
	/// /// __attribute__ ((visibility ("hidden"))) 
#endif
static map<string, enum PAN_PHASE_STATUS> map_pan_phase_status_name_pan_phase_status = pan_create_pan_phase_status_name_pan_phase_status_map();

static enum PAN_PHASE_STATUS pan_get_pan_phase_status(string& var_name)
{
	string s=var_name;
	transform(s.begin(),s.end(),s.begin(),(int(*)(int)) (int(*)(int)) toupper);

	auto it=map_pan_phase_status_name_pan_phase_status.begin();
	for (; it!=map_pan_phase_status_name_pan_phase_status.end(); ++it) {
		int L1 = (int)(*it).first.length();
		int L2 = (int)s.length();
		if (L1>L2) L1 = L2;
		if ((*it).first.compare(0, L1, s) == 0) break;
	}

	return (*it).second;
}

////////////////////////////////////////

enum PAN_UNIT {
	// temperature
	P_T_K,
	P_T_C,
	P_T_F,

	// pressure
	P_P_pascal,
	P_P_GPa,
	P_P_atm,
	P_P_bar,

	// mass amount
	P_M_mole_fraction,
	P_M_mole_percent,
	P_M_mole,

	P_M_weight_fraction,
	P_M_weight_percent,
	P_M_kg,

	P_M_FRACTION,
	P_M_PERCENT,

	// energy
	P_E_J,

	// time
	P_TIME_sec,
	P_TIME_min,
	P_TIME_hour,

	// length
	P_L_m,
	P_L_mm,
	P_L_um,
	P_L_nm,
	P_L_A,

	P_UNIT_NOT_DEFINED
};

const char PAN_UNIT_NAME[][max_num_chars] = {
	"K",
	"C",
	"F",

	"Pascal",
	"GPa",
	"atm",
	"bar",


	"x",
	"x%",
	"mole",

	"w",
	"w%",
	"kg",

	"fraction",
	"percent",

	"J",

	"second",
	"minute",
	"hour",

	"m",
	"mm",
	"um",
	"nm",
	"A"
};


//////////////  map <string, PAN_UNIT>  //////////////////////
static vector<pair<string, enum PAN_UNIT>> pan_create_pan_unit_vector()
{
 	vector<pair<string, enum PAN_UNIT>> v;

	int i;

	int size = sizeof(PAN_UNIT_NAME)/(max_num_chars * sizeof(char));

	for (i=0; i<size; i++)
	{
		string var_name = PAN_UNIT_NAME[i];
		transform(var_name.begin(), var_name.end(), var_name.begin(), (int(*)(int)) (int(*)(int)) toupper);
		v.push_back(make_pair(var_name, (PAN_UNIT)i));
	}

	return v;
}

static map<string, enum PAN_UNIT> pan_create_pan_unit_name_pan_unit_map()
{
 	map<string, enum PAN_UNIT> m;

	vector<pair<string, enum PAN_UNIT>> v = pan_create_pan_unit_vector();
	for (auto it=v.begin(); it!=v.end(); it++)
	{
		m[(*it).first] = (*it).second;
	}
	return m;
}

#ifdef __GNUC__ 
	/// /// __attribute__ ((visibility ("hidden"))) 
#endif
static map<string, enum PAN_UNIT> map_pan_unit_name_pan_unit = pan_create_pan_unit_name_pan_unit_map();

static enum PAN_UNIT pan_get_pan_unit(string& var_name)
{
	string s=var_name;
	transform(s.begin(),s.end(),s.begin(),(int(*)(int)) (int(*)(int)) toupper);

	auto it=map_pan_unit_name_pan_unit.begin();
	for (; it!=map_pan_unit_name_pan_unit.end(); ++it) {
		int L1 = (int)(*it).first.length();
		int L2 = (int)s.length();
		if (L1>L2) L1 = L2;
		if ((*it).first.compare(0, L1, s) == 0) break;
	}

	return (*it).second;
}

//////////////  vector <string> pan_var_list //////////////////////
static vector<string> pan_create_pan_var_list()
{
 	vector<string> v;

	v.push_back("f_tot(");
	v.push_back("fw_tot(");
	v.push_back("x_tot(");
	v.push_back("w_tot(");

	v.push_back("H_tot(");
	v.push_back("H_latent(");

	// physical properties
	v.push_back("alpha_Vm(");
	v.push_back("a_Vm(");	// same as alpha_Vm
	v.push_back("Vm(");
	v.push_back("density(");
	v.push_back("surface_tension(");
	v.push_back("viscosity(");
	v.push_back("conductivity(");
	v.push_back("valence(");

	v.push_back("Tc(");

	v.push_back("dTdx(");
	v.push_back("dxdT(");
	v.push_back("dwdT(");

	v.push_back("dxdT_L(");
	v.push_back("dwdT_L(");
	v.push_back("dxdT_S(");
	v.push_back("dwdT_S(");
	v.push_back("HSN(");
	v.push_back("eVal(");
	v.push_back("eVec(");

	v.push_back("fl(");
	v.push_back("fs(");
	v.push_back("f(");
	v.push_back("fv(");
	v.push_back("fw(");

	v.push_back("TPDF(");

	v.push_back("DF(");

	v.push_back("n_kg(");
	v.push_back("n_mole(");

	v.push_back("ma(");	// maximum
	v.push_back("mi(");	// minimum

	v.push_back("M(");
	v.push_back("DC(");
	v.push_back("DT(");
	v.push_back("ThF(");

	// other properties
	v.push_back("Qx(");
	v.push_back("Qw(");

	v.push_back("G(");
	v.push_back("H(");
	v.push_back("S(");
	v.push_back("Cp(");
	v.push_back("G_id(");
	v.push_back("H_id(");
	v.push_back("S_id(");
	v.push_back("G_ex(");
	v.push_back("H_ex(");
	v.push_back("S_ex(");

	v.push_back("mu(");
	v.push_back("a(");
	v.push_back("r(");
	v.push_back("rw(");

	v.push_back("y(");
	v.push_back("dGdy(");
	v.push_back("P(");

	v.push_back("x(");
	v.push_back("w(");
	v.push_back("n(");

	v.push_back("x0(");
	v.push_back("w0(");
	v.push_back("n0(");
	v.push_back("xs(");	// surface tenstion molar fraction

	v.push_back("struct(");

	return v;
}

#ifdef __GNUC__ 
	/// /// __attribute__ ((visibility ("hidden"))) 
#endif
static vector<string> pan_var_list = pan_create_pan_var_list();

////////////////////////////////////////
enum VAR_TYPE {
	VAR_NOT_DEFINED,	// not defined
	VAR_T,	// temperature
	VAR_P,	// external pressure
	VAR_x,	// mole fraction
	VAR_n,	// mass amount
	VAR_a,	// activity
	VAR_mu,	// chemical potential
	VAR_logP	// log10P of partial pressure
};

class Pan_Variable {
public:
	Pan_Variable(void) {
		m_type=VAR_NOT_DEFINED;
		m_value=0.0;
		m_name="";
		m_ref_phase="";
	}
	virtual ~Pan_Variable(void){
	}
	Pan_Variable(const Pan_Variable& rhs) {
		m_type=rhs.m_type;
		m_value=rhs.m_value;
		m_name=rhs.m_name;
		m_ref_phase=rhs.m_ref_phase;
	}
	Pan_Variable& operator=(const Pan_Variable& rhs) {
		if (this != &rhs) {
			m_type=rhs.m_type;
			m_value=rhs.m_value;
			m_name=rhs.m_name;
			m_ref_phase=rhs.m_ref_phase;
		}
		return *this;
	}
	bool operator == (const Pan_Variable &rhs) const {
		if (m_type != rhs.m_type) return false;
		if (m_name != rhs.m_name) return false;
		if (fabs(m_value - rhs.m_value) > 0.0001) return false;
		if (m_ref_phase != rhs.m_ref_phase) return false;
		return true;
	}
	bool operator != (const Pan_Variable &rhs) const {
		return !(*this==rhs);
	}

	friend Pan_Variable operator* (const Pan_Variable &d, double e) {
		Pan_Variable prod(d);
		prod.m_value = d.m_value*e;
		return prod;
	}
	friend Pan_Variable operator* (const Pan_Variable &d, int e) {
		Pan_Variable prod(d);
		prod.m_value = d.m_value*e;
		return prod;
	}

	friend Pan_Variable operator* (double e, const Pan_Variable &d) {
		Pan_Variable prod(d);
		prod.m_value = d.m_value*e;
		return prod;
	}
	friend Pan_Variable operator* (int e, const Pan_Variable &d) {
		Pan_Variable prod(d);
		prod.m_value = d.m_value*e;
		return prod;
	}

	friend Pan_Variable operator/ (const Pan_Variable &d, double e) {
		Pan_Variable q(d);
		q.m_value = d.m_value/e;
		return q;
	}
	friend Pan_Variable operator/ (const Pan_Variable &d, int e) {
		Pan_Variable q(d);
		q.m_value = d.m_value/e;
		return q;
	}

	friend Pan_Variable operator+(const Pan_Variable& c, const Pan_Variable& d) {
		Pan_Variable sum(c);
		sum.m_value += d.m_value;
		return sum;
	}
	friend Pan_Variable operator-(const Pan_Variable& c, const Pan_Variable& d) {
		Pan_Variable diff(c);
		diff.m_value -= d.m_value;
		return diff;
	}

public:
	VAR_TYPE m_type;
	string m_name;	// species name
	double m_value;
	string m_ref_phase;	// reference phase name

public:
		
	void reset() {
		m_type=VAR_NOT_DEFINED;
		m_value=0.0;
		m_name="";
		m_ref_phase="";
	}

	void print_debug(ostream& out) const {
		out << "$ [" << m_name << "] ";
		switch (m_type) {
		case VAR_T :
			out << "T";
			break;
		case VAR_P :
			out << "P";
			break;
		case VAR_x :
			out << "x";
			break;
		case VAR_n :
			out << "n";
			break;
		case VAR_a :
			out << "a";
			break;
		case VAR_mu :
			out << "mu";
			break;
		case VAR_logP :
			out << "logP";
			break;
		case VAR_NOT_DEFINED:
		default:
			break;
		}
		if (!m_ref_phase.empty()) {
			out << "(:" << m_ref_phase << ")";
		} 
		out <<  "=" << m_value;

	};
};

class pbfxBase
{
public:
	virtual void to_pbfx(char* file_name) const{;}
	virtual void to_pbfx(ostream& out, int tab=0) const{;}
	virtual string parse_pbfx(XMLNode* xNode){ return "";}
};

class Pan_Unit : public pbfxBase
{
public:

	Pan_Unit(){;}
	virtual ~Pan_Unit(){;}

public:
	string m_name;
	PAN_UNIT m_unit;
};


class Pan_Database  : public pbfxBase
{
public:
	Pan_Database(){ m_append = false;}
	virtual ~Pan_Database(){;}

public:
	string m_type;
	string m_file_name;
	bool m_append;
	string m_type_append_to;
	string m_file_name_append_to;
};

class Pan_Phase : public pbfxBase
{
public:
	Pan_Phase(){;}
	virtual ~Pan_Phase(){;}

public:
	string m_name;
	PAN_PHASE_STATUS m_status;
	string m_type;
};

class Pan_Reference_State : public pbfxBase
{
public:
	Pan_Reference_State(){;}
	virtual ~Pan_Reference_State(){;}
public:
	string m_name;
};

class Pan_Component : public pbfxBase
{
public:
	Pan_Component(){;}
	virtual ~Pan_Component(){;}

public:
	string m_name;
	PAN_STATUS m_status;
};

class Pan_Statespace : public pbfxBase
{
public:
	Pan_Statespace(){ m_balance_comp=""; }
	virtual ~Pan_Statespace(){;}

public:
	double m_T;
	double m_P;
	string m_T_expr;
	string m_P_expr;
	string m_balance_comp;
	map <string, double> m_n;
	map <string, string> m_n_expr;
	map <string, double> m_mu;
	map <string, double> m_logP;
	map <string, Pan_Variable> m_var_map;
	map <string, tuple<double, double, double>> m_range;
//	map <string, double> m_prop;

};

class Pan_PhasePoint : public pbfxBase
{
public:
	Pan_PhasePoint(){;}
	virtual ~Pan_PhasePoint(){;}

	void Add_y(string name, int s, int value)
	{
		m_y.push_back(make_tuple(name, s, (double)value));
	}

public:
	string m_name;
	vector<tuple<string, int, double>> m_y;
};

class Pan_Point;

class Pan_Region : public pbfxBase
{
public:
	Pan_Region() {
		m_name="region";
		m_num_grids=100;
		m_Vm=7.09e-006;
		m_length=0.001;
		m_vf = 0;
		m_interface_size=0;
		m_concentration_profile="";
	}
	virtual ~Pan_Region(){;}

public:
	string m_name;
	int m_num_grids;
	double m_Vm;	// molar volumn
	double m_length;
	double m_vf; // volume fraction

	double m_interface_size;
	string m_concentration_profile;

public:
//	vector<shared_ptr<Pan_Statespace>> m_st;
	vector<shared_ptr<Pan_Point>> m_points;
	vector <shared_ptr<Pan_Phase>> m_phases;
};

class Pan_Cell : public pbfxBase
{
public:
	Pan_Cell(){
		m_name="cell";
		m_type="alloy";
		m_active=true;

		m_num_grids = 100;

		m_geometry = "plate";
		m_inner_radius = 0;

		m_lower_boundary_type = "fixed_composition";
		m_upper_boundary_type="fixed_composition";

		m_lower_boundary_value = "";
		m_upper_boundary_value = "";


		m_left_boundary = "free"; 
		m_right_boundary = "free";


		m_solvent="";

		m_oxidation = make_pair("", "");
	}
	virtual ~Pan_Cell(){;}

public:
	string m_name;
	string m_type;
	bool m_active;

	int m_num_grids;

	string m_geometry;
	double m_inner_radius;

	string m_lower_boundary_type;
	string m_upper_boundary_type;

	
	string m_lower_boundary_value;
	string m_upper_boundary_value;

	// to be removed
	string m_left_boundary;
	string m_right_boundary;

	string m_solvent;
	pair<string, string> m_oxidation;
	pair<double, double> m_cosp;	// first = Kp; second = Q0

public:
	vector <shared_ptr<Pan_Region>> m_region;
};

class Pan_InitialValue : public pbfxBase
{
public:
	Pan_InitialValue(){;}
	virtual ~Pan_InitialValue(){;}

public:
	map <string, double> m_mu;
	vector <shared_ptr<Pan_PhasePoint>> m_phasepoints;
};

class Pan_Point : public pbfxBase
{
public:
	Pan_Point()
	{
		m_statespace = make_shared<Pan_Statespace>();
		m_initialvalue = make_shared<Pan_InitialValue>();
	}
	virtual ~Pan_Point(){;}

public:
	shared_ptr<Pan_Statespace> m_statespace;
	shared_ptr<Pan_InitialValue> m_initialvalue;
	vector<string> m_constraint;
};

class Pan_ThermalHistory : public pbfxBase
{
public:
	Pan_ThermalHistory(){;}
	virtual ~Pan_ThermalHistory(){;}

public:
	vector <pair<double, double>> m_node;
};

class Pan_Contour : public pbfxBase
{
public:
	Pan_Contour(){m_interval=0; m_val=0; m_start=make_pair(false, 0); m_stop=make_pair(false, 0); m_step=0;}
	virtual ~Pan_Contour(){;}

public:
	string m_name;
	
	string m_property;
	vector<string> m_constraint;

	pair<bool, double> m_start;
	pair<bool, double> m_stop;
	double m_step;

	double m_interval;
	double m_val;
	vector<string> m_phase;
};

class Pan_Diffusion : public pbfxBase
{
public:
	Pan_Diffusion(){
		m_name="diff"; 
		m_duration=10;
		m_max_time_step=0.1;
		m_relative_tolerance =1e-5;
		m_num_of_profile_outputs=1; 
		m_interface_flux_model="local_equilibrium";
		m_effective_mobility_model="HS_lower";	// HS_lower, HS_upper
		m_integral_settings = "automatic";
		m_Runge_Kutta_order = 1;
	}

	//Pan_Diffusion(const Pan_Diffusion& rhs){
	//	m_name=rhs.m_name;
	//	m_duration=rhs.m_duration;;
	//	m_max_time_step=rhs.m_max_time_step;
	//	m_num_of_profile_outputs=rhs.m_num_of_profile_outputs; 
	//	m_interface_flux_model=rhs.m_interface_flux_model;
	//	m_effective_mobility_model=rhs.m_effective_mobility_model;
	//}

	virtual ~Pan_Diffusion(){;}

public:
	string m_name;
	double m_duration;
	double m_max_time_step;
	double m_relative_tolerance;

	int m_num_of_profile_outputs;

	string m_interface_flux_model;
	string m_effective_mobility_model;
	vector<double> m_profiles;	// time at which concentration profiles are output

	string m_integral_settings;
	int m_Runge_Kutta_order;
};

class Pan_Precipitation : public pbfxBase
{
public:
	Pan_Precipitation() {
		m_name = "precipitation";
		m_T_freeze = -100;
		m_target_volume_percent = -1;
	}



	virtual ~Pan_Precipitation() { ; }

public:
	string m_name;
	// target volume percent for ttt simulation
	double m_target_volume_percent;
	// freezing temperature for precipitation simulation
	double m_T_freeze;

	vector<double> m_profiles;	// time at which psd profiles are output
};

class Pan_Condition : public pbfxBase
{
public:
	Pan_Condition()
	{
		m_steps_X=-1;
		m_steps_Y=-1;
		m_loops = -1;
		m_cur_loop = -1;

		m_scanline_density = 0;
		m_extensive_search = false;
		m_pseudo = false;
		m_with_subsystems = false;

		m_thermal_history = make_shared<Pan_ThermalHistory>();
		m_equilibriumType = "";
		m_equilibriumPhaseStatus = P_PHASE_ENTERED;

		// for solidification
		m_startFromLiquidusSurface =  true; /// start from liquidus surface;
		m_individual_phase = false;
		m_useAverageCompositionInSolidForScheil = true;

		m_model = "";
		m_coordinate = 0;
#ifdef PANENGINE
		m_step_T_max = 4;
#else
		m_step_T_max = 20;
#endif 
		m_step_T_min = 0.01;
		m_step_time_max = 20;
		m_step_time_min = 0.1;
		m_liquid_amount_min = 0.005;
		m_liquid_lamda_min = 0.8;
		m_x_min = 1e-12;	// when x[j]<m_min_x, component j is rejected
		m_primary_phase = "";
		m_secondary_phase = "";
		m_secondary_dendrite_arm_spacing = 0;	// in meter
		m_coarsening_constant = 0;	// coarsening constant

		m_solidification_rate = 50;
		m_T_gradient = 50;
		m_interfacial_energy = 0.093;
		m_latent_heat = 9.5e8;
		m_coarsening_geometric_factor = 35;
		m_solute_trapping_parameter = 1;
		m_sound_velocity = 1000;
		m_dendrite_tip_factor = 1;
		m_solid_diffusivity_factor = 1;
		m_boundary_layer_factor = 1;

		m_driving_force = false;
		m_thermo_property = true;
		m_physical_property = false;
		m_kinetic_property = false;
		m_user_property = false;
		m_phase_diagram = true;
		m_T_down = true;
		m_thermo_factor = false;
		m_Hessian = false;
		m_solvent = "";

		m_T_end = 300;
		m_T_end = 0;

		m_initial_condition_file = "";
		m_target_volume_percent = -1;
		m_normalized = true;

		m_duration = -1;	// in hour
		m_time_step = -1;	// in second
		m_profile_outputs = -1;
		m_T = -1;	// in K
		m_P = -1;	// in pascal

		// for EBC
		m_coating_name = "";
		m_air_file_name = "";
		m_fuel_ratio_phi = -1;
		m_input.first = "";
		m_input.second = "";
		m_eta = 5.179e-4; // in g/(cm sec)
		m_v = 440;	// in cm/sec
		m_L = 2.5;	// in cm
		m_D = 0.287;	// in cm^2/sec

		m_T_freeze = -100;
	}

	virtual ~Pan_Condition(){;}

public:
	// model type: lever rule or Scheil
	string m_model;
	int m_coordinate; // 0 for cartesian, 1 for cylindrical, 2 for spherical

	bool m_startFromLiquidusSurface;
	bool m_endWhenNoMoreLiquid;
	bool m_useAverageCompositionInSolidForScheil;

	// for line calculation, consider all phases, or phase by phase
	bool m_individual_phase;

	double m_step_T_max;
	double m_step_T_min;

	double m_step_time_max;
	double m_step_time_min;

	// solidification and projection termination temperature
	double m_T_end;
	
	// freezing temperature for precipitation simulation
	double m_T_freeze;

	// solidification minimum liquid amount to stop, in molar fraction
	double m_liquid_amount_min;

	// solidification minimum liquid amount remained in each step, in molar fraction
	double m_liquid_lamda_min;

	// minimum x
	double m_x_min;

	string m_primary_phase;
	string m_secondary_phase;
	double m_secondary_dendrite_arm_spacing;
	double m_coarsening_constant;

	// solidification simulation parameters
	double m_solidification_rate;
	double m_T_gradient;
	double m_interfacial_energy;
	double m_latent_heat;
	double m_coarsening_geometric_factor;
	double m_solute_trapping_parameter;
	double m_sound_velocity;
	double m_dendrite_tip_factor;
	double m_solid_diffusivity_factor;
	double m_boundary_layer_factor;

	// calculation steps for 1D and 2D calculation
	int m_steps_X;	// X steps
	int m_steps_Y;	// Y steps

	int m_loops;	// number of loops
	int m_cur_loop;	// current loop

	// scanline density, how many scanlines
	int m_scanline_density;

	// PanEngine search level: normal search and extensive search
	bool m_extensive_search;

	// calculation for m_pseudo binary 
	bool m_pseudo; 

	// liquidus projection with all subsystems or not
	bool m_with_subsystems;

	// phase diagram calculation
	bool m_phase_diagram;	// default value is true

	// driving force calculation
	bool m_driving_force;	// default value is false

	// user property calculation
	bool m_user_property;	// default value is false

	// thermodynamic property calculation
	bool m_thermo_property;	// default value is true

	// phyical property calculation
	bool m_physical_property;	// default value is false

	// kinetic property calculation
	bool m_kinetic_property;	// default value is false

	// calcualtion for optimizaer
	bool m_optimizer;	// default value is false

	string m_solvent;	// solvent component: for diffusion related property

	// equilibrium type
	string m_equilibriumType;

	// equilibrium phase status (entered, fixed) default:entered 
	PAN_PHASE_STATUS m_equilibriumPhaseStatus;

	// thermodynamic factor (ThF)
	bool m_thermo_factor;

	// Hessian (HSN, eVal, eVec)
	bool m_Hessian;

	// T decreasing
	bool m_T_down;

	// for precipitation simulation
	// the file name defining initial condition
	string m_initial_condition_file;

	// target volume percent for ttt simulation
	double m_target_volume_percent;
	bool m_normalized;

	// for diffusion
	double m_duration;	// in hour
	double m_time_step;	// in second
	int m_profile_outputs;	// number of profiles to output
	double m_T;	// temperature for diffusion & EBC
	double m_P;	// pressure for EBC

	// for EBC
	string m_coating_name;
	string m_air_file_name;	// store air species and partial pressure
	double m_fuel_ratio_phi;
	pair<string, string> m_input;	// first = source, second = value
	double m_eta; // in g/(cm sec)
	double m_v;	// in cm/sec
	double m_L;	// in cm
	double m_D;	// in cm^2/sec

// user will need use function to create the following pointers
public:
	shared_ptr<Pan_Diffusion> m_diff;
	shared_ptr<Pan_Precipitation> m_precipitation;
	shared_ptr<Pan_ThermalHistory> m_thermal_history;
	vector <shared_ptr<Pan_Contour>> m_contour;
	shared_ptr<Pan_Contour> m_cur_contour;
	
	vector<string> m_constraint;	// such as T=1000, P=1e5
	vector <pair<string, bool>> m_property;	// <type, value>

};

class Pan_Option : public pbfxBase
{
public:
	Pan_Option(){;}
	virtual ~Pan_Option(){;}
};

class Pan_System : public pbfxBase
{
public:
	Pan_System()
	{
		m_condition = make_shared<Pan_Condition>();
		m_option = make_shared<Pan_Option>();
		m_active = true;
		m_targetPhase = "liquid";

		set<string> temp;
		temp.insert("liquid");
		m_target_phases.push_back(make_pair(temp, 1.0));
	}
	virtual ~Pan_System(){;}

public:
	string m_name;
	bool m_active;
	vector <shared_ptr<Pan_Component>> m_components;
	vector <shared_ptr<Pan_Phase>> m_phases;
	vector <shared_ptr<Pan_Point>> m_points;
	vector <shared_ptr<Pan_Cell>> m_cells;
	vector <pair<string, shared_ptr<Pan_Database>>> m_databases;

	vector <string> m_alloys;
	string m_targetPhase;

	vector<pair<set<string>, double>> m_target_phases;

public:
	shared_ptr<Pan_Condition> m_condition;
	shared_ptr<Pan_Option> m_option;
};

class Pan_Table : public pbfxBase
{
public:
	Pan_Table(){;}
	virtual ~Pan_Table(){;}

public:
	string m_name;
	string m_source;
	string m_type;
	vector <string> column_name;
};

class Pan_Plot : public pbfxBase
{
public:
	Pan_Plot(){;}
	virtual ~Pan_Plot(){;}

public:
	string m_table_name;
	string m_table_path;
	string m_xaxis;
	string m_yaxis;
	string m_zaxis;
};

class Pan_Graph : public pbfxBase
{
public:
	Pan_Graph(){;}
	virtual ~Pan_Graph(){;}

public:
	string m_name;
	string m_type;
	vector <shared_ptr<Pan_Plot>> m_plots;
};

class Pan_Output : public pbfxBase
{
public:
	Pan_Output(){;}
	virtual ~Pan_Output(){;}

public:
	bool getTableName(int iTable, string & name)
	{
		if (iTable < 0 || iTable >= (int)m_tables.size())
		{
			return false;
		}
		int i = 0;
		vector <shared_ptr<Pan_Table>>::iterator it;

		for(auto it = m_tables.begin(); it!= m_tables.end(); it++)
		{
			if (i++==iTable)
			{
				name = it->get()->m_name;
				return true;
			}
		}

		return false;
	}

public:
	string m_unit;
	vector <shared_ptr<Pan_Table>> m_tables;
	vector <shared_ptr<Pan_Graph>> m_graphs;
};

class Pan_Calculation : public pbfxBase
{
public:
	Pan_Calculation()
	{
		m_calclation_ID=0;
		m_output = make_shared<Pan_Output>();
	}

	virtual ~Pan_Calculation(){;}

public:
	pair<string, string> m_appilcation_name_version;
	string m_name;
	//string m_type;
	PAN_CALC_TYPE m_calc_type;

	int m_calclation_ID;	// will be returned from PanPhaseDiagram
	vector <pair<string, shared_ptr<Pan_Database>>> m_databases;
	map <string, shared_ptr<Pan_Unit>> m_units;

public:
	vector<shared_ptr<Pan_System>> m_system;
	shared_ptr<Pan_Output> m_output;
};


class Solidification_Parameter;
class Solidification_Node;

typedef bool (*Pan_Progress)(char* msg, Solidification_Parameter&, vector<Solidification_Node>&);

enum PAN_PROP_LIST {
	PL_f,
	PL_fw,
	PL_n,	// amount in mole
	PL_G,
	PL_H,
	PL_S,
	PL_Cp,
	PL_G_ex,
	PL_H_ex,
	PL_S_ex,
	PL_DF,
	PL_x,
	PL_w,
	PL_mu,
	PL_Vm,
	PL_HSN
};

enum PAN_CALC_PROP {
	PC_ALL,	// all properties

	PC_THERMO,	// thermodynamic properties
		PC_H_S, // enthalpy and entropy
		PC_Cp_Partial,	// heat capacity and partial H and S
		PC_ThF,	// thermodynamic factor
		PC_HSN,	// Hessian, eigenvalue and eigenvector

	PC_DF, // driving force

	PC_KINETIC, // kinetic properties
		PC_M,	// mobility
		PC_D,	// diffusivity

	PC_PHYSICAL,	// physical property
		PC_Vm,
		PC_Viscosity,
		PC_Surface_Tension,
		PC_Conductivity,

	PC_USER_DEFINED,	// user-defined property

	PC_Dummy
};

enum PAN_PROP_FORMAT {
	P_PAN_TABLE,	// format of Pandat Table Columns
	P_POP	// format of POP file for optimization
};
