/////////////////////////////////////
//                                 //
//         PanEngine API           //
//                                 //
//         Version 2020            //
//        CompuTherm, LLC          //
//       Copyright, 2012-2020      //
//                                 //
//        www.computherm.com       //
//                                 //
/////////////////////////////////////

#pragma once
#include "stl.h"
#include "Pan_Global.h"

class P_Reference_State {
public:
	P_Reference_State(){m_name=""; m_G=m_H=m_S=0;}
	virtual ~P_Reference_State(){;}
	P_Reference_State(const P_Reference_State& rhs) {
		m_name = rhs.m_name;
		m_G = rhs.m_G;
		m_H = rhs.m_H;
		m_S = rhs.m_S;
	}
	P_Reference_State(shared_ptr<P_Reference_State> rhs) {
		m_name = rhs->m_name;
		m_G = rhs->m_G;
		m_H = rhs->m_H;
		m_S = rhs->m_S;
	}
	P_Reference_State& operator=(const P_Reference_State& rhs) {
		if (this != &rhs) {
			m_name = rhs.m_name;
			m_G = rhs.m_G;
			m_H = rhs.m_H;
			m_S = rhs.m_S;
		}
		return *this;
	}

public:
	string m_name;
	double m_G, m_H, m_S;

public:
	void print_debug(ostream& out) const
	{
		out	<< "m_name=" << m_name << "  m_G=" << m_G <<"  m_H=" << m_H << "  m_S=" << m_S;
		out << endl;
	}
};

class P_Component {
public:
	//P_Component(){m_n_fixed=true; m_n_unit=P_M_mole_fraction; m_active = true; m_slope=0; m_slope_w=0;m_valueDefined=-1;}
	//P_Component(string name){m_name=name; m_n_fixed=true; m_n_unit=P_M_mole_fraction; m_active = true; m_slope=0; m_slope_w=0;m_valueDefined=-1;}
	P_Component(){
		m_x = 1; m_w = 1; m_mol = 1; m_kg = 1; m_mu = 0;
		m_n_fixed = true; m_n_unit = P_M_mole_fraction; m_active = true; m_slope = 0; m_slope_w = 0; m_dxdT = 0; m_dwdT = 0; m_dummy = 0;
		m_dxdT_L = m_dxdT_S = m_dwdT_L = m_dwdT_S = 0;
	}
	P_Component(string name){
		m_name=name;
		m_x = 1; m_w = 1; m_mol = 1; m_kg = 1; m_mu = 0;
		m_n_fixed=true; m_n_unit=P_M_mole_fraction; m_active = true; m_slope=0; m_slope_w=0; m_dxdT = 0; m_dwdT = 0; m_dummy=0;
		m_dxdT_L = m_dxdT_S = m_dwdT_L = m_dwdT_S = 0;
	}
	virtual ~P_Component(){;}
	P_Component(const P_Component& cmp) {
		m_name = cmp.m_name;
		m_id = cmp.m_id;
		m_status = cmp.m_status;
		m_active = cmp.m_active;
		m_n_fixed = cmp.m_n_fixed;
		m_n_unit=cmp.m_n_unit;
		m_x = cmp.m_x;
		m_w = cmp.m_w;
		m_mol = cmp.m_mol;
		m_kg = cmp.m_kg;
		m_mu = cmp.m_mu;
		m_h = cmp.m_h;
		m_s = cmp.m_s;

		m_slope = cmp.m_slope;
		m_slope_w = cmp.m_slope_w;
		m_dxdT = cmp.m_dxdT;
		m_dwdT = cmp.m_dwdT;

		m_dxdT_L = cmp.m_dxdT_L;
		m_dxdT_S = cmp.m_dxdT_S;
		m_dwdT_L = cmp.m_dwdT_L;
		m_dwdT_S = cmp.m_dwdT_S;

		m_ref_st = cmp.m_ref_st;
		m_dummy = cmp.m_dummy;
	}
	P_Component(shared_ptr<P_Component> cmp_ptr) {
		m_name = cmp_ptr->m_name;
		m_id = cmp_ptr->m_id;
		m_status = cmp_ptr->m_status;
		m_active = cmp_ptr->m_active;
		m_n_fixed = cmp_ptr->m_n_fixed;
		m_n_unit=cmp_ptr->m_n_unit;
		m_x = cmp_ptr->m_x;
		m_w = cmp_ptr->m_w;
		m_mol = cmp_ptr->m_mol;
		m_kg = cmp_ptr->m_kg;
		m_mu = cmp_ptr->m_mu;
		m_h = cmp_ptr->m_h;
		m_s = cmp_ptr->m_s;

		m_slope = cmp_ptr->m_slope;
		m_slope_w = cmp_ptr->m_slope_w;
		m_dxdT = cmp_ptr->m_dxdT;
		m_dwdT = cmp_ptr->m_dwdT;

		m_dxdT_L = cmp_ptr->m_dxdT_L;
		m_dxdT_S = cmp_ptr->m_dxdT_S;
		m_dwdT_L = cmp_ptr->m_dwdT_L;
		m_dwdT_S = cmp_ptr->m_dwdT_S;	

		m_ref_st = cmp_ptr->m_ref_st;
		m_dummy = cmp_ptr->m_dummy;
	}
	P_Component& operator=(const P_Component& rhs) {
		if (this != &rhs) {
			m_name = rhs.m_name;
			m_id = rhs.m_id;
			m_status = rhs.m_status;
			m_active = rhs.m_active;
			m_n_fixed = rhs.m_n_fixed;
			m_n_unit = rhs.m_n_unit;
			m_x = rhs.m_x;
			m_w = rhs.m_w;
			m_mol = rhs.m_mol;
			m_kg = rhs.m_kg;
			m_mu = rhs.m_mu;
			m_h = rhs.m_h;
			m_s = rhs.m_s;

			m_slope = rhs.m_slope;
			m_slope_w = rhs.m_slope_w;
			m_dxdT = rhs.m_dxdT;
			m_dwdT = rhs.m_dwdT;

			m_dxdT_L = rhs.m_dxdT_L;
			m_dxdT_S = rhs.m_dxdT_S;
			m_dwdT_L = rhs.m_dwdT_L;
			m_dwdT_S = rhs.m_dwdT_S;

			m_ref_st = rhs.m_ref_st;
			m_dummy = rhs.m_dummy;
		}
		return *this;
	}

public:
	string m_name;
	int m_id;	// component ID

	PAN_STATUS m_status;
	bool m_active;

	PAN_UNIT m_n_unit;	// P_M_mole_fraction, P_M_mole_percent, P_M_mole
						// P_M_weight_fraction, P_M_weight_percent, P_M_kg

	double m_x;	// molar fraction
	double m_w;	// weight fraction

	double m_mol;	// amount in mole
	double m_kg;	// amount in kg

	double m_mu;	// chemical potential
	double m_h;		// enthalpy
	double m_s;		// entropy

	bool m_n_fixed;	// true: amount fixed    false: mu fixed

	double m_slope;	// dT/dx
	double m_slope_w;	// dT/dw
	double m_dxdT;	// dx/dT
	double m_dwdT;	// dw/dT

	double m_dxdT_L;	// for L in L+solid: dx/dT
	double m_dxdT_S;	// for S in L+solid: dx/dT
	double m_dwdT_L;	// for L in L+solid: dw/dT
	double m_dwdT_S;	// for S in L+solid: dw/dT

	double m_prop;	// temporary property
	int m_dummy;
//	double m_HSN;	// Hessian

	P_Reference_State m_ref_st;

public:
	friend P_Component operator+(const P_Component& rhs, const P_Component& lhs) {
		P_Component sum;

		sum.m_name = rhs.m_name;
		sum.m_id = rhs.m_id;
		sum.m_status = rhs.m_status;
		sum.m_active = rhs.m_active;
		sum.m_x = rhs.m_x+lhs.m_x;
		sum.m_w = rhs.m_w+lhs.m_w;
		sum.m_mol = rhs.m_mol+lhs.m_mol;
		sum.m_kg = rhs.m_kg+lhs.m_kg;

		sum.m_status = rhs.m_status;
		sum.m_n_fixed = rhs.m_n_fixed;
		sum.m_n_unit = rhs.m_n_unit;

		sum.m_mu = rhs.m_mu+lhs.m_mu;
		sum.m_h = rhs.m_mu+lhs.m_h;
		sum.m_s = rhs.m_s+lhs.m_s;

		return sum;
	}
	friend P_Component operator-(const P_Component& rhs, const P_Component& lhs) {
		P_Component diff;

		diff.m_name = rhs.m_name;
		diff.m_id = rhs.m_id;
		diff.m_status = rhs.m_status;
		diff.m_active = rhs.m_active;
		diff.m_x = rhs.m_x-lhs.m_x;
		diff.m_w = rhs.m_w-lhs.m_w;
		diff.m_mol = rhs.m_mol-lhs.m_mol;
		diff.m_kg = rhs.m_kg-lhs.m_kg;

		diff.m_status = rhs.m_status;
		diff.m_n_fixed = rhs.m_n_fixed;
		diff.m_n_unit = rhs.m_n_unit;

		diff.m_mu = rhs.m_mu-lhs.m_mu;
		diff.m_h = rhs.m_h-lhs.m_h;
		diff.m_s = rhs.m_s-lhs.m_s;

		return diff;
	}
	friend P_Component operator-(const P_Component& lhs) {
		P_Component diff;

		diff.m_name = lhs.m_name;
		diff.m_id = lhs.m_id;
		diff.m_status = lhs.m_status;
		diff.m_active = lhs.m_active;
		diff.m_x = -lhs.m_x;
		diff.m_w = -lhs.m_w;
		diff.m_mol = -lhs.m_mol;
		diff.m_kg = -lhs.m_kg;

		diff.m_status = lhs.m_status;
		diff.m_n_fixed = lhs.m_n_fixed;
		diff.m_n_unit = lhs.m_n_unit;

		diff.m_mu = -lhs.m_mu;
		diff.m_h = -lhs.m_h;
		diff.m_s = -lhs.m_s;

		return diff;
	}
	friend P_Component operator* (const P_Component &rhs, double e) {
		P_Component prod;

		prod.m_name = rhs.m_name;
		prod.m_id = rhs.m_id;
		prod.m_status = rhs.m_status;
		prod.m_active = rhs.m_active;
		prod.m_x = rhs.m_x*e;
		prod.m_w = rhs.m_w*e;
		prod.m_mol = rhs.m_mol*e;
		prod.m_kg = rhs.m_kg*e;

		prod.m_status = rhs.m_status;
		prod.m_n_fixed = rhs.m_n_fixed;
		prod.m_n_unit = rhs.m_n_unit;

		prod.m_mu = rhs.m_mu*e;
		prod.m_h = rhs.m_h*e;
		prod.m_s = rhs.m_s*e;
		return prod;
	}
	friend P_Component operator* (const P_Component &rhs, int e) {
		P_Component prod;

		prod.m_name = rhs.m_name;
		prod.m_id = rhs.m_id;
		prod.m_status = rhs.m_status;
		prod.m_active = rhs.m_active;
		prod.m_x = rhs.m_x*e;
		prod.m_w = rhs.m_w*e;
		prod.m_mol = rhs.m_mol*e;
		prod.m_kg = rhs.m_kg*e;

		prod.m_status = rhs.m_status;
		prod.m_n_fixed = rhs.m_n_fixed;
		prod.m_n_unit = rhs.m_n_unit;

		prod.m_mu = rhs.m_mu*e;
		prod.m_h = rhs.m_h*e;
		prod.m_s = rhs.m_s*e;

		return prod;
	}
	friend P_Component operator* (double e, const P_Component &lhs) {
		P_Component prod;

		prod.m_name = lhs.m_name;
		prod.m_id = lhs.m_id;
		prod.m_status = lhs.m_status;
		prod.m_active = lhs.m_active;
		prod.m_x = lhs.m_x*e;
		prod.m_w = lhs.m_w*e;
		prod.m_mol = lhs.m_mol*e;
		prod.m_kg = lhs.m_kg*e;

		prod.m_status = lhs.m_status;
		prod.m_n_fixed = lhs.m_n_fixed;
		prod.m_n_unit = lhs.m_n_unit;

		prod.m_mu = lhs.m_mu*e;
		prod.m_h = lhs.m_h*e;
		prod.m_s = lhs.m_s*e;

		return prod;
	}
	friend P_Component operator* (int e, const P_Component &lhs) {
		P_Component prod;

		prod.m_name = lhs.m_name;
		prod.m_id = lhs.m_id;
		prod.m_status = lhs.m_status;
		prod.m_active = lhs.m_active;
		prod.m_x = lhs.m_x*e;
		prod.m_w = lhs.m_w*e;
		prod.m_mol = lhs.m_mol*e;
		prod.m_kg = lhs.m_kg*e;

		prod.m_status = lhs.m_status;
		prod.m_n_fixed = lhs.m_n_fixed;
		prod.m_n_unit = lhs.m_n_unit;

		prod.m_mu = lhs.m_mu*e;
		prod.m_h = lhs.m_h*e;
		prod.m_s = lhs.m_s*e;

		return prod;
	}
	friend P_Component operator/ (const P_Component &rhs, double e) {
		P_Component q;

		q.m_name = rhs.m_name;
		q.m_id = rhs.m_id;
		q.m_status = rhs.m_status;
		q.m_active = rhs.m_active;
		q.m_x = rhs.m_x/e;
		q.m_w = rhs.m_w/e;
		q.m_mol = rhs.m_mol/e;
		q.m_kg = rhs.m_kg/e;

		q.m_status = rhs.m_status;
		q.m_n_fixed = rhs.m_n_fixed;
		q.m_n_unit = rhs.m_n_unit;

		q.m_mu = rhs.m_mu/e;
		q.m_h = rhs.m_h/e;
		q.m_s = rhs.m_s/e;

		return q;
	}
	friend P_Component operator/ (const P_Component &rhs, int e) {
		P_Component q;

		q.m_name = rhs.m_name;
		q.m_id = rhs.m_id;
		q.m_status = rhs.m_status;
		q.m_active = rhs.m_active;
		q.m_x = rhs.m_x/e;
		q.m_w = rhs.m_w/e;
		q.m_mol = rhs.m_mol/e;
		q.m_kg = rhs.m_kg/e;

		q.m_status = rhs.m_status;
		q.m_n_fixed = rhs.m_n_fixed;
		q.m_n_unit = rhs.m_n_unit;

		q.m_mu = rhs.m_mu/e;
		q.m_h = rhs.m_h/e;
		q.m_s = rhs.m_s/e;

		return q;
	}

public:
	void print_debug(ostream& out) const
	{
		out << "(" << PAN_UNIT_NAME[m_n_unit] << ")" << " [" << m_name << (m_active?"":"*") << "," << m_id << "] x" 
			<< "=" << m_x << " w=" << m_w
			<< " mu" << (m_n_fixed?"":"$") << "=" << m_mu 
			<< " mol=" << m_mol << " kg=" << m_kg;	
#if defined(_PEDEB) || defined(_PPDDEB)
		out << " H=" << m_h << " S=" << m_s;
#endif
		out << " dxdT_L=" << m_dxdT_L;
		out << " dxdT_S=" << m_dxdT_S;
		out << " dwdT_L=" << m_dwdT_L;
		out << " dwdT_S=" << m_dwdT_S;
#if defined(_PEDEB) || defined(_PPDDEB)
		out	<< "  slope=" << m_slope;
		out	<< "  slopew=" << m_slope_w;
		out << "  dxdT=" << m_dxdT;
		out << "  dwdT=" << m_dwdT;
#endif
		if (!m_ref_st.m_name.empty()) {
			out << "Ref. St.[" << m_ref_st.m_name << "] ";
			out << "G[" << m_ref_st.m_G << "] ";
			out << "H[" << m_ref_st.m_H << "] ";
			out << "S[" << m_ref_st.m_S << "] ";
		}
		out << endl;
	}

	bool valid() {
		if (m_x < -1e-10) return false;
		if (1-m_x < -1e-10) return false;
		return true;
	}

	bool valid(double min_x) {
		return (m_x>=min_x);
	}
};

class P_Species
{
public:
	P_Species(void) {
		m_name="";
		m_n_fixed=true;
		m_status=P_SELECTED;
		m_q=0.0;
	}
	P_Species(string& nm) {
		m_name=nm;
		m_n_fixed=true;
		m_status=P_SELECTED;
		m_q=0.0;
	}
	virtual ~P_Species(void){
	}
	P_Species(const P_Species& rhs) {
		m_name=rhs.m_name;
		m_status=rhs.m_status;
		m_q=rhs.m_q;
		m_c=rhs.m_c;
		m_n_fixed=rhs.m_n_fixed;
		m_n=rhs.m_n;
		m_var=rhs.m_var;
		m_mu=rhs.m_mu;
		m_H=rhs.m_H;
		m_S=rhs.m_S;
	}
	P_Species& operator=(const P_Species& rhs) {
		if (this != &rhs) {
			m_name=rhs.m_name;
			m_status=rhs.m_status;
			m_q=rhs.m_q;
			m_c=rhs.m_c;
			m_n_fixed=rhs.m_n_fixed;
			m_n=rhs.m_n;
			m_mu=rhs.m_mu;
			m_H=rhs.m_H;
			m_S=rhs.m_S;
			m_var=rhs.m_var;
		}

		return *this;
	}

public:
	string m_name;
	PAN_STATUS m_status;
	double m_q;	// charge
	double m_atom; // amount of atoms:  m_atom=3 for Al2Cu   m_atom=2 for Al2Va
	vector<pair<string, double>> m_c;	// first: component name
										// second: amount of components
	vector<pair<shared_ptr<P_Component>, double>> m_comp;

public:
	bool m_n_fixed;	// true: m_n is given; false: m_mu or other variable is given
	double m_n;

	double m_mu;
	double m_H;
	double m_S;

	Pan_Variable m_var;

public:
	bool operator == (const P_Species &rhs) const {
		if (m_n_fixed != rhs.m_n_fixed) return false;
		if (m_n_fixed) {
			if (fabs(m_n - rhs.m_n) > 0.0001) return false;
		} else {
			return (m_var==rhs.m_var);
		}

		return true;
	}
	friend P_Species operator* (const P_Species &d, double e) {
		P_Species prod(d);
		if (prod.m_n_fixed) {
			prod.m_n *= e;
		} else {
			prod.m_var.m_value *= e;
		}
		return prod;
	}
	friend P_Species operator* (const P_Species &d, int e) {
		P_Species prod(d);
		if (prod.m_n_fixed) {
			prod.m_n *= e;
		} else {
			prod.m_var.m_value *= e;
		}
		return prod;
	}

	friend P_Species operator* (double e, const P_Species &d) {
		P_Species prod(d);
		if (prod.m_n_fixed) {
			prod.m_n *= e;
		} else {
			prod.m_var.m_value *= e;
		}
		return prod;
	}
	friend P_Species operator* (int e, const P_Species &d) {
		P_Species prod(d);
		if (prod.m_n_fixed) {
			prod.m_n *= e;
		} else {
			prod.m_var.m_value *= e;
		}
		return prod;
	}

	friend P_Species operator/ (const P_Species &d, double e) {
		P_Species q(d);
		if (q.m_n_fixed) {
			q.m_n /= e;
		} else {
			q.m_var.m_value /= e;
		}
		return q;
	}
	friend P_Species operator/ (const P_Species &d, int e) {
		P_Species q(d);
		if (q.m_n_fixed) {
			q.m_n /= e;
		} else {
			q.m_var.m_value /= e;
		}
		return q;
	}

	friend P_Species operator+(const P_Species& c, const P_Species& d) {
		P_Species sum(c);
		if (sum.m_n_fixed) {
			sum.m_n += d.m_n;
		} else {
			sum.m_var.m_value += d.m_var.m_value;
		}

		return sum;
	}
	friend P_Species operator-(const P_Species& c, const P_Species& d) {
		P_Species diff(c);

		if (diff.m_n_fixed) {
			diff.m_n -= d.m_n;
		} else {
			diff.m_var.m_value -= d.m_var.m_value;
		}

		return diff;
	}
	friend P_Species operator-(const P_Species& d) {
		P_Species diff(d);

		if (diff.m_n_fixed) {
			diff.m_n = -d.m_n;
		} else {
			diff.m_var.m_value = -d.m_var.m_value;
		}

		return diff;
	}
public:
	void set_name(string nm) {m_name = nm;}
	string get_name() {return m_name;}

	void set_q(double q) {m_q = q;}
	double get_q() {return m_q;};

	void set_status(PAN_STATUS stat) {m_status=stat;};

	void set_component(vector<pair<string, double>>& sp) {
		m_c.resize(0);
		m_c = sp;
	}
	vector<pair<string,double>> get_component() {
		return m_c;
	}

public:
	void set_n(double n) {
		m_n_fixed = true;
		m_n = n;
	}
	double get_n() {
		return m_n;
	}

	bool get_n_fixed() {
		return m_n_fixed;
	}


public:
	void print_debug(ostream& out) const {
		m_var.print_debug(out);
		out << " mu=" << m_mu << endl;
	}
};

class P_Statespace {
public:
	P_Statespace(): m_mass_unit(0) {
		m_T = 298.15;
		m_P = 101325;
	}
	virtual ~P_Statespace(){;}
	P_Statespace(const P_Statespace& st) {
		m_T_fixed = st.m_T_fixed;
		m_T = st.m_T;

		m_P_fixed = st.m_P_fixed;
		m_P = st.m_P;

		m_mass_unit = st.m_mass_unit;

		m_comp.clear();

        for(auto it = st.m_comp.begin(); it!=st.m_comp.end(); it++)
        {
			m_comp[it->first] = make_shared<P_Component>(it->second);

        }
		if (st.m_species.size()>0) {
			m_species.clear();
			for(auto it=st.m_species.begin(); it!=st.m_species.end(); it++)
			{
				shared_ptr<P_Species> sp_ptr = make_shared<P_Species>(*(*it).second);
				m_species.insert(make_pair((*it).first, sp_ptr));
			}
		}
		for (auto it = st.m_prop.begin(); it != st.m_prop.end(); it++)
		{
			m_prop[it->first] = it->second;
		}
		m_var_map = st.m_var_map;
	}

	P_Statespace(shared_ptr<P_Statespace> st_ptr) {
		m_T_fixed = st_ptr->m_T_fixed;
		m_T = st_ptr->m_T;

		m_P_fixed = st_ptr->m_P_fixed;
		m_P = st_ptr->m_P;

		m_mass_unit = st_ptr->m_mass_unit;

		map<string, shared_ptr<P_Component>>::const_iterator it;
		m_comp.clear();
        for(it =st_ptr->m_comp.begin(); it!=st_ptr->m_comp.end(); it++)
        {
			m_comp[it->first] = make_shared<P_Component>(it->second);
        }

		if (st_ptr->m_species.size()>0) {
			m_species.clear();
			for(auto it=st_ptr->m_species.begin(); it!=st_ptr->m_species.end(); it++)
			{
				shared_ptr<P_Species>sp_ptr = make_shared<P_Species>(*((*it).second));
				m_species.insert(make_pair((*it).first, sp_ptr));
			}
		}
		for (auto it = st_ptr->m_prop.begin(); it != st_ptr->m_prop.end(); it++)
		{
			m_prop[it->first] = it->second;
		}
		m_var_map = st_ptr->m_var_map;
	}
	P_Statespace& operator=(const P_Statespace& rhs) {
		if (this != &rhs) {
			m_T_fixed = rhs.m_T_fixed;
			m_T = rhs.m_T;

			m_P_fixed = rhs.m_P_fixed;
			m_P = rhs.m_P;

			m_mass_unit = rhs.m_mass_unit;

			m_comp.clear();
			m_species.clear();

			map<string, shared_ptr<P_Component>>::const_iterator it;

            for(it =rhs.m_comp.begin(); it!=rhs.m_comp.end(); it++)
            {
				m_comp[it->first] = make_shared<P_Component>(it->second);
            }
			if (rhs.m_species.size()>0) {
				for(auto it=rhs.m_species.begin(); it!=rhs.m_species.end(); it++)
				{
					m_species.insert(make_pair((*it).first, (*it).second));
				}
			}
			for (auto it = rhs.m_prop.begin(); it != rhs.m_prop.end(); it++)
			{
				m_prop[it->first] = it->second;

			}
			m_var_map = rhs.m_var_map;
		}
		return *this;
	}

	void set_from(shared_ptr<P_Statespace> st) {
		// m_st has all components in st, if m_st
		m_T_fixed = st->m_T_fixed;
		m_T = st->m_T;
		m_P_fixed = st->m_P_fixed;
		m_P = st->m_P;
		m_mass_unit = st->m_mass_unit;

		for(auto it=m_comp.begin(); it!=m_comp.end(); it++)
		{
			auto c=st->m_comp.find((*it).first);
			if (c!=st->m_comp.end()) {
				(*it).second = make_shared<P_Component>((*c).second);
			} else {
				(*it).second->m_x = 0;
				(*it).second->m_w = 0;
				(*it).second->m_mol = 0;
				(*it).second->m_kg = 0;
				(*it).second->m_mu = 0;
				(*it).second->m_h = 0;
				(*it).second->m_s = 0;
				(*it).second->m_slope = 0;
				(*it).second->m_slope_w = 0;
				(*it).second->m_dxdT = 0;
				(*it).second->m_dwdT = 0;
			}
		}
		m_species.clear();

		if (st->m_species.size()>0) {
			for(auto it=st->m_species.begin(); it!=st->m_species.end(); it++)
			{
				m_species.insert(make_pair((*it).first, (*it).second));
			}
		}
		if (st->m_prop.size() > 0) {
			for (auto it = st->m_prop.begin(); it != st->m_prop.end(); it++)
			{
				m_prop[it->first] = it->second;
			}
		}
		m_var_map = st->m_var_map;
	}

	bool operator == (const P_Statespace &rhs) const {
		if (fabs(m_T - rhs.m_T) > 0.0001) return false;
		if (fabs(m_P - rhs.m_P) > 0.01) return false;
		if (m_comp.size() != rhs.m_comp.size()) return false;
		bool eq=true;

		for(auto it =m_comp.begin(); it!=m_comp.end(); it++)
		{
			auto itr=rhs.m_comp.find(it->first);
			if (itr!=rhs.m_comp.end()) {
				if (fabs(it->second->m_x-itr->second->m_x) > 0.00001) {
					eq=false;
				}
			}
		}
		return eq;
	}
	bool operator != (const P_Statespace &rhs) const {
		return !(*this == rhs);
	}

public:
	bool m_T_fixed;
	double m_T;	// in K
	bool m_P_fixed;
	double m_P;	// in pascal

	int m_mass_unit;	// 0 = x; 1 = w; 2 = mol; 3 = kg
						// default is 0

	map<string, shared_ptr<P_Component>> m_comp;
	map<string, shared_ptr<P_Species>> m_species;
	map<string, tuple<double, double, double>> m_range;
	map<string, double> m_prop;
	map <string, Pan_Variable> m_var_map;
public:
	friend P_Statespace operator+(const P_Statespace& rhs, const P_Statespace& lhs) {
		P_Statespace sum;

		sum.m_T_fixed = rhs.m_T_fixed;
		sum.m_P_fixed = rhs.m_P_fixed;

		sum.m_T = rhs.m_T + lhs.m_T;
		sum.m_P = rhs.m_P * lhs.m_P;

        for(auto it=rhs.m_comp.begin(); it!=rhs.m_comp.end(); it++)
        {
            sum.m_comp.insert(pair<string, shared_ptr<P_Component>>(it->first, make_shared<P_Component>(*it->second)));
        }

        for(auto it=lhs.m_comp.begin(); it!=lhs.m_comp.end(); it++)
        {
			 auto itr=sum.m_comp.find(it->first);
			 if (itr!=sum.m_comp.end()) {
				 *(*itr).second = *(*itr).second + *(*it).second;
			 } else {
				sum.m_comp.insert(pair<string, shared_ptr<P_Component>>(it->first, make_shared<P_Component>(*it->second)));
			 }
        }

        for(auto it=rhs.m_species.begin(); it!=rhs.m_species.end(); it++)
        {
            sum.m_species.insert(pair<string, shared_ptr<P_Species>>(it->first, make_shared<P_Species>(*it->second)));
        }

        for(auto it=lhs.m_species.begin(); it!=lhs.m_species.end(); it++)
        {
			 auto itr=sum.m_species.find(it->first);
			 if (itr!=sum.m_species.end()) {
				 *(*itr).second = *(*itr).second + *(*it).second;
			 } else {
				sum.m_species.insert(pair<string, shared_ptr<P_Species>>(it->first, make_shared<P_Species>(*it->second)));
			 }
        }

		for (auto it = lhs.m_prop.begin(); it != lhs.m_prop.end(); it++)
		{
			auto itr = sum.m_prop.find(it->first);
			if (itr != sum.m_prop.end()) {
				itr->second = itr->second + it->second;
			}
			else {
				sum.m_prop.insert(pair<string, double>(it->first, it->second));
			}
		}

		for (auto it = lhs.m_var_map.begin(); it != lhs.m_var_map.end(); it++)
		{
			auto itr = sum.m_var_map.find(it->first);
			if (itr != sum.m_var_map.end()) {
				itr->second = itr->second + it->second;
			}
			else {
				sum.m_var_map[it->first] = it->second;
			}
		}
		return sum;
	}
	friend P_Statespace operator-(const P_Statespace& rhs, const P_Statespace& lhs) {
		P_Statespace diff;

		diff.m_T_fixed = rhs.m_T_fixed;
		diff.m_P_fixed = rhs.m_P_fixed;

		diff.m_T = rhs.m_T - lhs.m_T;
		diff.m_P = rhs.m_P / lhs.m_P;

        for(auto it=rhs.m_comp.begin(); it!=rhs.m_comp.end(); it++)
        {
            diff.m_comp.insert(pair<string, shared_ptr<P_Component>>(it->first, make_shared<P_Component>(*it->second)));
        }

        for(auto it=lhs.m_comp.begin(); it!=lhs.m_comp.end(); it++)
        {
			 auto itr=diff.m_comp.find(it->first);
			 if (itr!=diff.m_comp.end()) {
				 *(*itr).second = *(*itr).second - *(*it).second;
			 } else {
				diff.m_comp.insert(pair<string, shared_ptr<P_Component>>(it->first, make_shared<P_Component>(-*it->second)));
			 }
        }

        for(auto it=rhs.m_species.begin(); it!=rhs.m_species.end(); it++)
        {
            diff.m_species.insert(pair<string, shared_ptr<P_Species>>(it->first, make_shared<P_Species>(*it->second)));
        }

        for(auto it=lhs.m_species.begin(); it!=lhs.m_species.end(); it++)
        {
			 auto itr=diff.m_species.find(it->first);
			 if (itr!=diff.m_species.end()) {
				 *(*itr).second = *(*itr).second - *(*it).second;
			 } else {
				diff.m_species.insert(pair<string, shared_ptr<P_Species>>(it->first, make_shared<P_Species>(-*it->second)));
			 }
        }

		for (auto it = lhs.m_prop.begin(); it != lhs.m_prop.end(); it++)
		{
			auto itr = diff.m_prop.find(it->first);
			if (itr != diff.m_prop.end()) {
				itr->second = itr->second - it->second;
			}
			else {
				diff.m_prop.insert(pair<string, double>(it->first, it->second));
			}
		}

		for (auto it = lhs.m_var_map.begin(); it != lhs.m_var_map.end(); it++)
		{
			auto itr = diff.m_var_map.find(it->first);
			if (itr != diff.m_var_map.end()) {
				itr->second = itr->second - it->second;
			}
			else {
				diff.m_var_map[it->first] = it->second;
			}
		}
		return diff;
	}
	friend P_Statespace operator* (const P_Statespace &rhs, double e) {
		P_Statespace prod;

		prod.m_T_fixed = rhs.m_T_fixed;
		prod.m_P_fixed = rhs.m_P_fixed;

		prod.m_T = rhs.m_T * e;;
		prod.m_P = exp(log(rhs.m_P) * e);

        for(auto it=rhs.m_comp.begin(); it!=rhs.m_comp.end(); it++)
        {  
			prod.m_comp[it->first] = make_shared<P_Component>(*it->second * e);
        }

        for(auto it=rhs.m_species.begin(); it!=rhs.m_species.end(); it++)
        {  
			prod.m_species[it->first] = make_shared<P_Species>(*it->second * e);
        }

		for (auto it = rhs.m_prop.begin(); it != rhs.m_prop.end(); it++)
		{
			prod.m_prop[it->first] = it->second * e;
		}

		for (auto it = rhs.m_var_map.begin(); it != rhs.m_var_map.end(); it++)
		{
			prod.m_var_map[it->first] = it->second * e;
		}
		return prod;
	}
	friend P_Statespace operator* (const P_Statespace &rhs, int e) {
		P_Statespace prod;

		prod.m_T_fixed = rhs.m_T_fixed;
		prod.m_P_fixed = rhs.m_P_fixed;

		prod.m_T = rhs.m_T * e;
		prod.m_P = exp(log(rhs.m_P) * e);

        for(auto it =rhs.m_comp.begin(); it!=rhs.m_comp.end(); it++)
        {    
			prod.m_comp[it->first] = make_shared<P_Component>(*it->second * e);
        }

        for(auto it=rhs.m_species.begin(); it!=rhs.m_species.end(); it++)
        {  
			prod.m_species[it->first] = make_shared<P_Species>(*it->second * e);
        }

		for (auto it = rhs.m_prop.begin(); it != rhs.m_prop.end(); it++)
		{
			prod.m_prop[it->first] = it->second * e;
		}

		for (auto it = rhs.m_var_map.begin(); it != rhs.m_var_map.end(); it++)
		{
			prod.m_var_map[it->first] = it->second * e;
		}
		return prod;
	}
	friend P_Statespace operator* (double e, const P_Statespace &lhs) {
		P_Statespace prod;

		prod.m_T_fixed = lhs.m_T_fixed;
		prod.m_P_fixed = lhs.m_P_fixed;

		prod.m_T = lhs.m_T * e;
		prod.m_P = exp(log(lhs.m_P) * e);

        for(auto it =lhs.m_comp.begin(); it!=lhs.m_comp.end(); it++)
        {  
			prod.m_comp[it->first] = make_shared<P_Component>(*it->second * e);
        }

        for(auto it=lhs.m_species.begin(); it!=lhs.m_species.end(); it++)
        {  
			prod.m_species[it->first] = make_shared<P_Species>(*it->second * e);
        }

		for (auto it = lhs.m_prop.begin(); it != lhs.m_prop.end(); it++)
		{
			prod.m_prop[it->first] = it->second * e;
		}

		for (auto it = lhs.m_var_map.begin(); it != lhs.m_var_map.end(); it++)
		{
			prod.m_var_map[it->first] = it->second * e;
		}
		return prod;
	}
	friend P_Statespace operator* (int e, const P_Statespace &lhs) {
		P_Statespace prod;

		prod.m_T_fixed = lhs.m_T_fixed;
		prod.m_P_fixed = lhs.m_P_fixed;

		prod.m_T = lhs.m_T * e;
		prod.m_P = exp(log(lhs.m_P) * e);

		map<string, shared_ptr<P_Component>>::const_iterator it;

        for(auto it=lhs.m_comp.begin(); it!=lhs.m_comp.end(); it++)
        {  
			prod.m_comp[it->first] = make_shared<P_Component>(*it->second * e);
        }

        for(auto it=lhs.m_species.begin(); it!=lhs.m_species.end(); it++)
        {  
			prod.m_species[it->first] = make_shared<P_Species>(*it->second * e);
        }

		for (auto it = lhs.m_prop.begin(); it != lhs.m_prop.end(); it++)
		{
			prod.m_prop[it->first] = it->second * e;
		}

		for (auto it = lhs.m_var_map.begin(); it != lhs.m_var_map.end(); it++)
		{
			prod.m_var_map[it->first] = it->second * e;
		}
		return prod;
	}
	friend P_Statespace operator/ (const P_Statespace &rhs, double e) {
		P_Statespace q;

		q.m_T_fixed = rhs.m_T_fixed;
		q.m_P_fixed = rhs.m_P_fixed;

		q.m_T = rhs.m_T / e;
		q.m_P = rhs.m_P * exp(1.0/e);

        for(auto it=rhs.m_comp.begin(); it!=rhs.m_comp.end(); it++)
        {              
			q.m_comp[it->first] = make_shared<P_Component>(*it->second / e);
        }

        for(auto it=rhs.m_species.begin(); it!=rhs.m_species.end(); it++)
        {              
			q.m_species[it->first] = make_shared<P_Species>(*it->second / e);
        }

		for (auto it = rhs.m_prop.begin(); it != rhs.m_prop.end(); it++)
		{
			q.m_prop[it->first] = it->second / e;
		}

		for (auto it = rhs.m_var_map.begin(); it != rhs.m_var_map.end(); it++)
		{
			q.m_var_map[it->first] = it->second / e;
		}
		return q;
	}
	friend P_Statespace operator/ (const P_Statespace &rhs, int e) {
		P_Statespace q;

		q.m_T_fixed = rhs.m_T_fixed;
		q.m_P_fixed = rhs.m_P_fixed;

		q.m_T = rhs.m_T / e;
		q.m_P = rhs.m_P * exp(1.0/e);

        for(auto it=rhs.m_comp.begin(); it!=rhs.m_comp.end(); it++)
        {    
			q.m_comp[it->first] = make_shared<P_Component>(*it->second / e);
        }

        for(auto it=rhs.m_species.begin(); it!=rhs.m_species.end(); it++)
        {              
			q.m_species[it->first] = make_shared<P_Species>(*it->second / e);
        }

		for (auto it = rhs.m_prop.begin(); it != rhs.m_prop.end(); it++)
		{
			q.m_prop[it->first] = it->second / e;
		}

		for (auto it = rhs.m_var_map.begin(); it != rhs.m_var_map.end(); it++)
		{
			q.m_var_map[it->first] = it->second / e;
		}
		return q;
	}

public:
	void print_debug(ostream& out) const
	{
		out << "T" << ((m_T_fixed)? "$":"") << "=" << m_T << " K  " << "  logP=" << log10(m_P) << "  P" << ((m_P_fixed)? "$":"") << "=" << m_P << " Pascal"  << endl;

        for(auto it =m_comp.begin(); it!=m_comp.end(); it++)
        {
			it->second->print_debug(out);
        }

		for (auto it = m_prop.begin(); it != m_prop.end(); it++)
		{
			out << it->first << " = " << it->second << endl;
		}

		if (m_species.size() > 0) {
			for(auto it =m_species.begin(); it!=m_species.end(); it++)
			{
				it->second->print_debug(out);
			//	out << endl;
			}
		}

		for (auto it = m_var_map.begin(); it != m_var_map.end(); it++)
		{
		//	out << (*it).first << "(" << (*it).second.m_name << ") = " << (*it).second.m_value << endl;
			out << (*it).first << " = " << (*it).second.m_value << endl;
		}
	}

	bool valid() {
		if (m_species.size() == 0) {
			for(auto c=m_comp.begin(); c!=m_comp.end(); ++c) {
				if (!(*c).second->valid()) {
					return false;
				}
			}
		}
		return true;
	}

	bool valid(double min_x) {
		if (m_species.size() == 0) {
			for(auto c=m_comp.begin(); c!=m_comp.end(); ++c) {
				if (!(*c).second->valid(min_x)) {
					return false;
				}
			}
		}
		return true;
	}
};
	
class P_Coord
{
public:
	P_Coord() {
		angle_factor = 1.0; 
		n_dim=2; 
		dr_min = 0.011/16.0; 
		dr_max = 0.02; 
		angle_tot = 0;
		pi = 3.14159265359;
		twoD = false;
		step_type = 0;
	};

	~P_Coord() {};

	P_Coord(const P_Coord& rhs) {
		u = rhs.u;
		uv = rhs.uv;

		st_Y = rhs.st_Y;
		st_O = rhs.st_O;
		st_X = rhs.st_X;

		st_X_O = rhs.st_X_O;
		st_Y_O = rhs.st_Y_O;

		uv_start = rhs.uv_start;
		uv_end = rhs.uv_end;
		dr = rhs.dr;
		step = rhs.step;
		angle_factor = rhs.angle_factor;
		u_fixed = rhs.u_fixed;
		n_dim = rhs.n_dim;
		twoD = rhs.twoD;
		fixed_length = rhs.fixed_length;
		step_type = rhs.step_type;
	};

	P_Coord& operator=(const P_Coord& rhs) {
		if (this != &rhs) {
			u = rhs.u;
			uv = rhs.uv;

			st_Y = rhs.st_Y;
			st_O = rhs.st_O;
			st_X = rhs.st_X;

			st_X_O = rhs.st_X_O;
			st_Y_O = rhs.st_Y_O;

			uv_start = rhs.uv_start;
			uv_end = rhs.uv_end;
			dr = rhs.dr;
			step = rhs.step;
			angle_factor = rhs.angle_factor;
			u_fixed = rhs.u_fixed;
			n_dim = rhs.n_dim;
			twoD = rhs.twoD;
			fixed_length = rhs.fixed_length;
			step_type = rhs.step_type;
		}
		return *this;
	};

public:
	P_Statespace st_Y, st_O, st_X;
	P_Statespace st_X_O, st_Y_O;
	// current scanline
	complex<double> uv_start;
	complex<double> uv_end;

public:
	// current start position in 2D
	complex<double> start_pos;

	// current direction in 2D
	complex<double> dr;
	double dr_min, dr_max;

	bool fixed_length;	// T: length part is fixed, angle part is changing.
						// F: angle part is fixed, length part is changing.
	bool twoD;
	int step_type;
public:
	double pi;
	double angle_tot;

public:
	int step;
	double angle_factor;

	bool u_fixed;
	int n_dim;	// dimention

private:
	double u;

	complex<double> uv;	// u & v
	bool same_statespace(shared_ptr<P_Statespace> X, shared_ptr<P_Statespace> Y) {
		if (fabs(X->m_T - Y->m_T) > 1e-6 || fabs(X->m_P - Y->m_P) > 1e-3) {
			return false;
		}
		for (auto i=X->m_comp.begin(); i!=X->m_comp.end(); ++i) {
			auto j=Y->m_comp.find((*i).first);
			if (j!=Y->m_comp.end()) {
				if (fabs((*i).second->m_x - (*j).second->m_x) > 1e-6) {
					return false;
				}
			}
		}
		return true;
	};
public:
	void set_st(shared_ptr<P_Statespace> O) {
		st_O = *O;
	};

	string set_st(shared_ptr<P_Statespace> O, shared_ptr<P_Statespace> X) {
		if (same_statespace(O, X)) {
			return "identical coordinate points";
		}

		st_O = *O;
		st_X = *X;

		st_X_O = st_X - st_O;
		st_Y_O = st_O - st_O;	// set as zero
		return "";
	};

	string set_st(shared_ptr<P_Statespace> Y, shared_ptr<P_Statespace> O, shared_ptr<P_Statespace> X) {
		if (same_statespace(Y, O) || same_statespace(Y, X) || same_statespace(O, X)) {
			return "identical coordinate points";
		}

		st_Y = *Y;
		st_O = *O;
		st_X = *X;

		st_X_O = st_X - st_O;
		st_Y_O = st_Y - st_O;
		return "";
	};

	void set_scanline(pair<double, double>& s, pair<double, double>& e) {
		uv_start = complex<double>(s.first, s.second);
		uv_end = complex<double>(e.first, e.second);
	}

	void set_scanline(complex<double> s, complex<double> e) {
		uv_start = s;
		uv_end = e;
	}

	void get_scanline(complex<double>& s, complex<double>& e) {
		s = uv_start;
		e = uv_end;
	}	
	
	void get_scanline(pair<double, double>& s, pair<double, double>& e) {
		s.first = uv_start.real();
		s.second = uv_start.imag();
		e.first = uv_end.real();
		e.second = uv_end.imag();
	}

	P_Statespace get_st(complex<double>& uv_) {
		uv = uv_;
		P_Statespace s;
		s =  st_O + uv.real() * st_X_O + uv.imag() * st_Y_O;
		for (auto c=s.m_comp.begin(); c!=s.m_comp.end(); ++c) {
			if ((*c).second->m_x < 0 && (*c).second->m_x > -1e-15) {
				(*c).second->m_x = 0;
			}
		}
		return s;
	};

	complex<double> get_uv(shared_ptr<P_Statespace> s) {
		double u, v;
		u = v = 0;
		bool found = false;
		double uu[2], vv[2], cc[2];
		int id = 0;

		if (fabs(st_X_O.m_T) > 1e-5 || fabs(st_Y_O.m_T) > 1e-5) {
			uu[id] = st_X_O.m_T;
			vv[id] = st_Y_O.m_T;
			cc[id] = s->m_T - st_O.m_T;
			id++;
		}
		auto cY = st_Y_O.m_comp.begin();
		for (auto cX = st_X_O.m_comp.begin(); cX != st_X_O.m_comp.end(); ++cX, ++cY) {
			if (fabs(cX->second->m_x) > 1e-7 || fabs(cY->second->m_x) > 1e-7) {
				uu[id] = cX->second->m_x;
				vv[id] = cY->second->m_x;
				auto c1 = s->m_comp.find(cX->first);
				auto cO = st_O.m_comp.find(cX->first);
				cc[id] = c1->second->m_x - cO->second->m_x;
				id++;
			}
			if (id>1) {
				break;
			}
		}

		if (id < 2) {
			if (fabs(st_X_O.m_P - 1) > 1e-5 || fabs(st_Y_O.m_P - 1) > 1e-5) {
				uu[id] = st_X_O.m_P;
				vv[id] = st_Y_O.m_P;
				cc[id] = s->m_P / st_O.m_P;
				id++;
			}
		}

		if (id == 2) {
			double D = uu[0] * vv[1] - uu[1] * vv[0];
			double Du = cc[0] * vv[1] - cc[1] * vv[0];
			double Dv = uu[0] * cc[1] - uu[1] * cc[0];
			u = Du / D;
			v = Dv / D;
		}
		return complex<double>(u, v);
	};

	P_Statespace get_st(double pos) {	// get position on the current scanline
		uv=uv_start + (uv_end - uv_start) * pos;
		return get_st(uv);
	};

	void set_start_pos(complex<double> uv) {
		start_pos = uv;
	};

	complex<double> get_start_pos() {
		return start_pos;
	};

	complex<double> get_uv_start() {
		return uv_start;
	};

	complex<double> get_uv_end() {
		return uv_end;
	};

	void set_dr(complex<double> r) {
		dr = r;
	};

	complex<double> get_dr() {
		return dr;
	};

	bool increase_dr(double fact=2) {
		if (abs(dr) > dr_max) return false;
		dr *= fact;
		return true;
	};

	bool decrease_dr(double fact=0.5) {
		if (abs(dr) < dr_min) return false;
		dr *= fact;
		return true;
	};

	bool set_dr_min_max(double min, double max) {
		if (fabs(min) > fabs(max)) return false;
		dr_min = fabs(min);
		dr_max = fabs(max);
		return true;
	};

	void set_fixed_length(bool b=true) {
		fixed_length = b;
	};

	void set_fixed_angle(bool b=true) {
		fixed_length = !b;
	};

	bool is_fixed_length() {
		return fixed_length;
	};

	bool is_fixed_angle() {
		return !fixed_length;
	};

	void set_uv(complex<double> uv_) {
		uv = uv_;
	};

	complex<double> get_uv() {
		return uv;
	};

	void set_u_fixed(bool fixed) {
		u_fixed = fixed;
	};

	bool get_u_fixed() {
		return u_fixed;
	};

	double get_pos(complex<double> c) {	// get position in one-D from uv
		double pos=0;
		complex<double> d_uv;

		d_uv = uv_end - uv_start;

		if (fabs(d_uv.real()) > 1e-10) {
			pos = (c.real() - uv_start.real()) / (uv_end.real() - uv_start.real());
		} else {
			pos = (c.imag() - uv_start.imag()) / (uv_end.imag() - uv_start.imag());
		}

		return pos;
	};

	void reset_total_angle(double angle = 0) {
		angle_tot = angle;
	};

	void add_to_total_angle(double angle) {
		angle_tot += angle;
		return;
	};

	double get_total_angle() {
		return angle_tot;
	};

	double is_full_circle() {
		return fabs(angle_tot) > 2 * pi; 
	};

public:
	void print_debug(ostream& out) {
		out << "start_pos = " << start_pos << "  step_type = " << step_type << endl;
		out << "dr = " << dr << " polar (" << abs(dr) << "," << arg(dr) << ")" << endl;
		out << "uv_start = " << uv_start << "   uv_end = " << uv_end << endl;
	};

	void print_debug_more(ostream& out) {
		out << "@@ st_Y : \n";
		st_Y.print_debug(out);
		out << "@@ st_O : \n";
		st_O.print_debug(out);
		out << "@@ st_X : \n";
		st_X.print_debug(out);
		out << "@@ st_X_O : \n";
		st_X_O.print_debug(out);
		out << "@@ st_Y_O : \n";
		st_Y_O.print_debug(out);
	};

};

struct icomp {
	bool operator() (const std::string& lhs, const std::string& rhs) const {
		return _stricmp(lhs.c_str(), rhs.c_str()) < 0;
	}
};

class Phase_Info {
public:
 	Phase_Info() {};
	~Phase_Info() {};
	Phase_Info(const Phase_Info &info) {
		ftot = info.ftot;
		fwtot = info.fwtot;
		comp_tot = info.comp_tot;
		residual_liq = false;
		Vm = info.Vm;
	}

public:
	double ftot;	// cumulative molar fraction of the phase
	double fwtot;	// cumulative weight fraction of the phase
	bool residual_liq;
	double Vm;	// cumulative molar volumn of the phase
	map<string, pair<double, double> > comp_tot;	// cumulatively average molar fraction (first) and weight fraction (second) of component in the phase
	//   comp_name     x       wt

public:
	void print_debug(ostream& out) {
		out << "  ftot = " << ftot << "  fwtot=" << fwtot << endl;
		for (auto c=comp_tot.begin(); c!=comp_tot.end(); ++c) {
			out << "  x[" << (*c).first << "]=" <<  (*c).second.first << "  w[" << (*c).first << "]=" <<  (*c).second.second << endl;
		}
	}
};

enum PROP_TYPE {
	p_Tc,
	p_BMag,
	p_Vm,
	p_alpha_Vm,
	p_Density,
	p_Surface_Tension,
	p_Viscosity,
	p_Conductivity,
	p_Valence,

	p_M,	// mobility
	p_DC,	// chemical diffusivity
	p_DT,	// trace diffusivity
	p_ThF,	// thermodynamic factor
	p_Qx	// growth-restriction parameter

};

const char PROP_NAME[][20] = {
	"Tc",
	"BMag",
	"Vm",
	"alpha_Vm",
	"Density",
	"Surface_Tension",
	"Viscosity",
	"Conductivity",
	"Valence",

	"M",
	"DC",
	"DT",
	"ThF",
	"Qx"
};

class Val {	// for value
public:
	Val() { m_exist = true; };
	Val(double v) {
		m_val=v;
	};
	~Val() {};
	Val(const Val &v) {
		m_val = v.m_val;
		m_comp = v.m_comp;
		m_grad_comp = v.m_grad_comp;
		m_ref_comp = v.m_ref_comp;
		m_exist = v.m_exist;
	}

	bool operator == (const Val &rhs) const {
	//	return (m_comp == rhs.m_comp && m_grad_comp == rhs.m_grad_comp && m_ref_comp == rhs.m_ref_comp);
		return (_stricmp(m_comp.c_str(), rhs.m_comp.c_str()) == 0
			&& _stricmp(m_grad_comp.c_str(), rhs.m_grad_comp.c_str()) == 0
			&& _stricmp(m_ref_comp.c_str(), rhs.m_ref_comp.c_str()) == 0);
	}

public:
	string m_comp;
	string m_grad_comp;
	string m_ref_comp;
	bool m_exist;
	double m_val;
};

class P_Position {
public:
	P_Position() {
		u=v=w=r=0;
	}
	~P_Position() {
	}
	P_Position(const P_Position& rhs) {
		u=rhs.u;
		v=rhs.v;
		w=rhs.w;
		r=rhs.r;
	}
public:
	double u, v, w, r;
};

////class P_Eigen {
////public:
////	P_Eigen() {};
////	~P_Eigen() {};
////	P_Eigen(const P_Eigen &prop) {
////		m_prop.clear();
////		m_prop = prop.m_prop;
////	}
////public:
////
////	bool get_value(int id, pair<double, map<string, double>>& val) const {
////		//  id: index of eigenvalue
////		if (id < 0 || id>= m_prop.size()) return false;
////		val = m_prop[id];
////		return true;
////	}
////
////	void print_debug(ostream& out) const {
////		for (auto it = m_prop.begin(); it != m_prop.end(); ++it) {
////			out << "EVal    " << (*it).first << endl;
////			out << "EVec    ";
////			for (auto v = (*it).second.begin(); v != (*it).second.end(); ++v) {
////				out << "<" << (*v).first << ", " << (*v).second << ">\n";
////				out << "        ";
////			}
////			out << endl;
////		}
////	}
////
////public:
////	//	map<PROP_TYPE, Val> m_prop;
////	//	map<pair<int, string>, Val> m_prop;
////	//map<string, Val> m_prop;
////	vector<pair<double, map<string, double>>> m_prop;
////	//           EVal       comp_name  x     
////	//                       EVector
////};

class P_Property {
public:
	P_Property() {};
	~P_Property() {};
	P_Property(const P_Property &prop) {
		m_prop.clear();
		for (auto p=prop.m_prop.begin(); p!=prop.m_prop.end(); ++p) {
			m_prop[(*p).first] = (*p).second;
		}
	}
public:

	bool exist(string str) const {
		if (m_prop.empty()) return false;
		for (auto i = m_prop.begin(); i != m_prop.end(); ++i) {
			if ((*i).first == str) {
				return true;
			}
		}
		return false;
	}

	bool get_value(string str, Val& val) const {

		for (auto i = m_prop.begin(); i != m_prop.end(); ++i) {
			if ((*i).first == str && (*i).second == val) {
				val.m_val = (*i).second.m_val;
				return true;
			}
		}
		val.m_exist = false;
		val.m_val = 0;
		return val.m_exist;
	}

	double get_value(const string& p_name_str) const {
		for (auto i=m_prop.begin(); i!=m_prop.end(); ++i) {
			if (p_name_str == (*i).first) {
				return (*i).second.m_val;
			}
		}
		return 0;
	}

	bool get_value(pair<string, double>& v) const {
		for (auto i = m_prop.begin(); i != m_prop.end(); ++i) {
			if (v.first == (*i).first) {
				v.second = (*i).second.m_val;
				return true;
			}
		}
		return false;
	}

	//double get_value(const int& p_name_int) const {
	//	for (auto i=m_prop.begin(); i!=m_prop.end(); ++i) {
	//		if (p_name_int == (*i).first.first) {
	//			return (*i).second.m_val;
	//		}
	//	}
	//	return 0;
	//}



	//void set_value(const int& p_name_int, double v) {
	//	pair<int, string> p_name = make_pair(p_name_int, "");
	//	m_prop[p_name] = v;
	//}

	void set_value(const string& p_name_str, double v) {
	//	pair<int, string> p_name = make_pair(-1, p_name_str);
		m_prop[p_name_str] = Val(v);
	}

	//void set_value(const pair<int, string>& p_name, double v) {
	//	m_prop[p_name] = v;
	//}

	void print_debug(ostream& out) const {
		for (auto it=m_prop.begin(); it!=m_prop.end(); ++it) {
		//	out << "    " << PROP_NAME[(*it).first];
			out << "    " << (*it).first;
			out << "=" << (*it).second.m_val << " ";

			if ((*it).first == "Tc") {
				out << "K";
			} else 
			if (_stricmp((*it).first.c_str(), "Vm") == 0) {
				out << "cm^3/mole";
			} else 
			if (_stricmp((*it).first.c_str(), "MW") == 0) {	// atomic/molecular weight
				out << "g/mole";
			} else
			if (_stricmp((*it).first.c_str(), "alpha_Vm") == 0) {
				out << "K^-1";
			} else 
			if (_stricmp((*it).first.c_str(), "Density") == 0) {
				out << "g/cm^3";
			} else 
			if (_stricmp((*it).first.c_str(), "Surface_Tension") == 0) {
				out << "N/m";
			} else 
			if (_stricmp((*it).first.c_str(), "Viscosity") == 0) {
				out << "Ns/m^2";
			} else 
			if (_stricmp((*it).first.c_str(), "Conductivity") == 0) {
				out << "S/m";
			} else 
			if (_stricmp((*it).first.c_str(), "BMag") == 0) {
				out << " ";
			} else {
				out << " ";
			}

/*			switch ((*it).first.first) {
			case (int)p_Tc:
				out << "K";
				break;
			case (int)p_Vm:
				out << "cm^3/mole";
				break;
			case (int)p_alpha_Vm:
				out << "K^-1";
				break;
			case (int)p_Density:
				out << "g/cm^3";
				break;
			case (int)p_Surface_Tension:
				out << "N/m";
				break;
			case (int)p_Viscosity:
				out << "Ns/m^2";
				break;
			case (int)p_Conductivity:
				out << "S/m";
				break;
			case (int)p_BMag:
				break;

			default:
				break;
			}
*/
			out << endl;
		}
	}

public:
//	map<PROP_TYPE, Val> m_prop;
//	map<pair<int, string>, Val> m_prop;
	map<string, Val> m_prop;
};

class P_Property_vector {
public:
	P_Property_vector() {};
	~P_Property_vector() {};
	P_Property_vector(const P_Property_vector &prop) {
		m_prop.clear();
		for (auto p=prop.m_prop.begin(); p!=prop.m_prop.end(); ++p) {
			m_prop.push_back(*p);
		}
	}
public:
	bool exist(string str) const {
		if (m_prop.empty()) return false;

		for (auto i = m_prop.begin(); i != m_prop.end(); ++i) {
			if ((*i).first == str) {
				return true;
			}
		}
		return false;
	}
	//bool get_value(PROP_TYPE p_name, Val& val) const {

	//	for (auto i=m_prop.begin(); i!=m_prop.end(); ++i) {
	//		if ((*i).second == (int)p_name && (*i).second==val) {
	//			val.m_val = (*i).second.m_val;
	//			return true;
	//		}
	//	}
	//	val.m_exist = false;
	//	val.m_val = 0;
	//	return val.m_exist;
	//}
	bool get_value(string str, Val& val) const {

		for (auto i=m_prop.begin(); i!=m_prop.end(); ++i) {
			if ((*i).first == str && (*i).second==val) {
				val.m_val = (*i).second.m_val;
				return true;
			}
		}
		val.m_exist = false;
		val.m_val = 0;
		return val.m_exist;
	}

		
	double get_value(string str) const {
		double v = 0;
		for (auto i=m_prop.begin(); i!=m_prop.end(); ++i) {
			if ((*i).first == str) {
				v = (*i).second.m_val;
				break;
			}
		}
		return v;
	}

	void remove(string str) {
		for (int i = m_prop.size()-1; i >= 0; --i) {
			if (m_prop[i].first == str) {
				m_prop.erase(m_prop.begin()+i);
			}
		}
		return;
	}
	//void set_value(PROP_TYPE p_name, Val& v) {
	//	m_prop.push_back(make_pair(p_name, v));
	//}

	void print_debug(ostream& out) const {
		for (auto it=m_prop.begin(); it!=m_prop.end(); ++it) {
		//	out << "    " << PROP_NAME[(*it).first];
			out << "    " << (*it).first;
			if ((*it).first == "M" || (*it).first == "DT") {
				out << "(" << (*it).second.m_comp << ")";
			} else 
			if ((*it).first == "xs") {
				out << "(" << (*it).second.m_comp << ")";
			}
			else
			if ((*it).first == "DC") {
				out << "(" << (*it).second.m_comp << "," << (*it).second.m_grad_comp << "@*:" << (*it).second.m_ref_comp << ")";
			} else
			if ((*it).first == "ThF") {
	//			out << "(" << (*it).second.m_comp << "," << (*it).second.m_grad_comp << "@*:" << (*it).second.m_ref_comp << ")";
				out << "(" << (*it).second.m_comp << "," << (*it).second.m_grad_comp << "@*)";
			}
			else
			if ((*it).first == "Qx") {

			} else {

			}

			//switch ((*it).first) {
			//case p_M:
			//case p_DT:
			//	out << "(" << (*it).second.m_comp << ")";
			//	break;
			//case p_DC:
			//	out << "(" << (*it).second.m_comp << "," << (*it).second.m_grad_comp << "@*:" << (*it).second.m_ref_comp << ")";
			//	break;
			//case p_ThF:
			//	out << "(" << (*it).second.m_comp << "," << (*it).second.m_grad_comp << "@*:" << (*it).second.m_ref_comp << ")";
			//	break;
			//case p_Qx:
			////	out << "(" << (*it).second.m_comp << "," << (*it).second.m_grad_comp << "@*:" << (*it).second.m_ref_comp << ")";
			//	break;
			//default:
			//	break;
			//}

			out << "=" << (*it).second.m_val << " ";

			if ((*it).first == "M") {
				out << "m^2 mole_atoms/s/J";
			} else
			if ((*it).first == "DC" || (*it).first == "DT") {
				out << "m^2/s";
			} else {

			}

			//switch ((*it).first) {
			//case p_M:
			//	out << "m^2 mole_atoms/s/J";
			//	break;
			//case p_DC:
			//case p_DT:
			//	out << "m^2/s";
			//	break;
			//default:
			//	break;
			//}
			out << endl;
		}
	}

public:
	vector<pair<string, Val>> m_prop;
};

class P_Phase_Model_Info {
public:
	P_Phase_Model_Info() { m_num_phases=1; m_seq_id=0; };
	P_Phase_Model_Info(string& name) { m_name = name; m_num_phases=1; m_seq_id=0; };
	P_Phase_Model_Info(string& name, int n) { m_name = name; m_num_phases=n; m_seq_id=0; };
	P_Phase_Model_Info(string& name, int n, int sid) { m_name = name; m_num_phases=n; m_seq_id=sid; };

	~P_Phase_Model_Info() {};

	P_Phase_Model_Info(const P_Phase_Model_Info& rhs) {
		m_name = rhs.m_name;
		m_id = rhs.m_id;
		m_num_phases = rhs.m_num_phases;
		m_seq_id = rhs.m_seq_id;
		m_comp = rhs.m_comp;
		m_constituent = rhs.m_constituent;
	};

public:
	string m_name;
	int m_id;	// phase id in database
	int m_num_phases;	// maximum number of this phase in a point if there is a miscibility gap
	int m_seq_id;	// for solidification, sequence ID of phases
	set <string> m_comp;
	vector< pair<double, vector<string>> > m_constituent;	// first=sublattice size, second=species vector
};

class P_2nd_order_Info {
public:
	P_2nd_order_Info() { m_has_2nd_order=false; };
	~P_2nd_order_Info() {};

	P_2nd_order_Info(const P_2nd_order_Info& rhs) {
		m_disorder_name = rhs.m_disorder_name;
		m_disorder_id = rhs.m_disorder_id;
		m_has_2nd_order = rhs.m_has_2nd_order;
		m_dy = rhs.m_dy;
	};

public:
	string m_disorder_name;	// name of disorder part
	int m_disorder_id;	// phase id in database of disorder part
	bool m_has_2nd_order;	// this point has 2nd order
	double m_dy;
};

class Hessian {
public: 
	double m_HSN;
	vector<pair<double, map <string, double>>> m_eigen;
	//         eigenvalue            eigenvector

	vector<vector<tuple <string, string, double>>> m_Hessian;
	//                     i        j   d2G/dxidxj

	void print_debug(ostream& out) const {
		out << "HSN = " << m_HSN << endl;
		out << "Hessian:" << endl;
		for (auto i = m_Hessian.begin(); i != m_Hessian.end(); ++i) {
			for (auto j = (*i).begin(); j != (*i).end(); ++j) {
				out << "   <" << get<0>(*j) << ", " << get<1>(*j) << ", "<< get<2>(*j) <<">  ";
			}
			out << endl;
		}
	//	out << endl;

		for (auto it = m_eigen.begin(); it != m_eigen.end(); ++it) {
			out << "   m_eigen = " << (*it).first;
			for (auto v = (*it).second.begin(); v != (*it).second.end(); ++v) {
				out << " <" << (*v).first << ", " << (*v).second << ">";
			}
			out << endl;
		}
	}
};

class P_Phase_Point
{
public:
	P_Phase_Point(){
		m_ph_nth = 0;
		m_phase_id = -1;
		m_status = P_PHASE_ENTERED;
		m_in_equilibrium = true;
		m_n_comp_per_formula = 0;
		m_mole = 0;
		m_kg = 0;
		m_f = 0;
		m_fw = 0;
		m_f_fixed = false;
		m_dummy = 0;
		m_dummy_str = "";
		m_Hess.first = false;

		m_2nd_order.first=false;

		m_magnetic.first=false;
		m_physical.first=false;
		m_kinetic.first=false;
		m_phase_property.first=false;
		m_thermodynamic.first=false;
		m_misc_property.first=false;
		m_G_ex = m_S_ex = m_Cp = 0;
	}

	virtual ~P_Phase_Point(){;}

	P_Phase_Point(const P_Phase_Point& ppt) {
		m_phase_name = ppt.m_phase_name;
		m_phase_id = ppt.m_phase_id;
		m_ph_nth = ppt.m_ph_nth;
		m_status = ppt.m_status;
		m_in_equilibrium = ppt.m_in_equilibrium;
		m_n_comp_per_formula = ppt.m_n_comp_per_formula;
		m_mole = ppt.m_mole;
		m_kg = ppt.m_kg;
		m_f = ppt.m_f;
		m_f_fixed = ppt.m_f_fixed;
		m_comp = ppt.m_comp;
		m_f = ppt.m_f;
		m_fw = ppt.m_fw;
		m_G = ppt.m_G;
		m_H = ppt.m_H;
		m_S = ppt.m_S;
		m_G_ex = ppt.m_G_ex;
		m_S_ex = ppt.m_S_ex;
		m_Cp = ppt.m_Cp;
		m_TPDF = ppt.m_TPDF;
		m_Hess = ppt.m_Hess;
		m_dummy = ppt.m_dummy;
		m_dummy_str = ppt.m_dummy_str;
		//m_dbl_dummy = ppt.m_dbl_dummy;
		//m_b_dummy = ppt.m_b_dummy;
		//m_str_dummy = ppt.m_str_dummy;
		m_2nd_order = ppt.m_2nd_order;

		m_magnetic = ppt.m_magnetic;
		m_thermodynamic = ppt.m_thermodynamic;
		m_physical = ppt.m_physical;
		m_kinetic = ppt.m_kinetic;
		m_phase_property = ppt.m_phase_property;
		m_misc_property = ppt.m_misc_property;

		m_st = make_shared<P_Statespace>(*ppt.m_st);

		m_y.clear();
        vector<shared_ptr<map <string, double>>>::const_iterator it;
		for(it = ppt.m_y.begin(); it!= ppt.m_y.end(); it++)
		{
			shared_ptr<map <string, double>> yy = make_shared<map <string, double>> ();
			*yy=**it;
			m_y.push_back(yy);
		}

		if (ppt.m_dGdy.size() != 0) {
			m_dGdy.clear();
			vector<shared_ptr<map <string, double>>>::const_iterator it;
			for (it = ppt.m_dGdy.begin(); it != ppt.m_dGdy.end(); it++)
			{
				shared_ptr<map <string, double>> eps = make_shared<map <string, double>>();
				*eps = **it;
				m_dGdy.push_back(eps);
			}
		}
		m_structure = ppt.m_structure;
	}

	P_Phase_Point(shared_ptr<P_Phase_Point> ppt_ptr) {
		m_phase_name = ppt_ptr->m_phase_name;
		m_phase_id = ppt_ptr->m_phase_id;
		m_ph_nth = ppt_ptr->m_ph_nth;
		m_status = ppt_ptr->m_status;
		m_in_equilibrium = ppt_ptr->m_in_equilibrium;
		m_n_comp_per_formula = ppt_ptr->m_n_comp_per_formula;
		m_mole = ppt_ptr->m_mole;
		m_kg = ppt_ptr->m_kg;
		m_f = ppt_ptr->m_f;
		m_f_fixed = ppt_ptr->m_f_fixed;
		m_comp = ppt_ptr->m_comp;
		m_f = ppt_ptr->m_f;
		m_fw = ppt_ptr->m_fw;
		m_G = ppt_ptr->m_G;
		m_H = ppt_ptr->m_H;
		m_G_ex = ppt_ptr->m_G_ex;
		m_S_ex = ppt_ptr->m_S_ex;
		m_S = ppt_ptr->m_S;
		m_Cp = ppt_ptr->m_Cp;
		m_TPDF = ppt_ptr->m_TPDF;
		m_Hess = ppt_ptr->m_Hess;
		m_dummy = ppt_ptr->m_dummy;
		m_dummy_str = ppt_ptr->m_dummy_str;
		//m_dbl_dummy = ppt_ptr->m_dbl_dummy;
		//m_b_dummy = ppt_ptr->m_b_dummy;
		//m_str_dummy = ppt_ptr->m_str_dummy;
		m_2nd_order = ppt_ptr->m_2nd_order;

		m_magnetic = ppt_ptr->m_magnetic;
		m_thermodynamic = ppt_ptr->m_thermodynamic;
		m_physical = ppt_ptr->m_physical;
		m_kinetic = ppt_ptr->m_kinetic;
		m_phase_property = ppt_ptr->m_phase_property;
		m_misc_property = ppt_ptr->m_misc_property;

		m_st = make_shared<P_Statespace>(*ppt_ptr->m_st);

		m_y.clear();
		vector<shared_ptr<map <string, double>>>::const_iterator it;
		for (it = ppt_ptr->m_y.begin(); it!=ppt_ptr->m_y.end(); it++)
		{
			shared_ptr<map <string, double>> yy = make_shared<map <string, double>> ();
			*yy=**it;
			m_y.push_back(yy);
		}

		if (ppt_ptr->m_dGdy.size() != 0) {
			m_dGdy.clear();
			vector<shared_ptr<map <string, double>>>::const_iterator it;
			for (it = ppt_ptr->m_dGdy.begin(); it != ppt_ptr->m_dGdy.end(); it++)
			{
				shared_ptr<map <string, double>> tem = make_shared<map <string, double>>();
				*tem = **it;
				m_dGdy.push_back(tem);
			}
		}
		m_structure = ppt_ptr->m_structure;
	}

	P_Phase_Point& operator=(const P_Phase_Point& rhs) {
		if (this != &rhs) {
			m_phase_name = rhs.m_phase_name;
			m_phase_id = rhs.m_phase_id;
			m_ph_nth = rhs.m_ph_nth;
			m_status = rhs.m_status;
			m_in_equilibrium = rhs.m_in_equilibrium;
			m_n_comp_per_formula = rhs.m_n_comp_per_formula;
			m_mole = rhs.m_mole;
			m_kg = rhs.m_kg;
			m_f = rhs.m_f;
			m_f_fixed = rhs.m_f_fixed;
			m_comp = rhs.m_comp;

			m_f = rhs.m_f;
			m_fw = rhs.m_fw;
			m_G = rhs.m_G;
			m_H = rhs.m_H;
			m_G_ex = rhs.m_G_ex;
			m_S_ex = rhs.m_S_ex;
			m_S = rhs.m_S;
			m_Cp = rhs.m_Cp;
			m_TPDF = rhs.m_TPDF;
			m_Hess = rhs.m_Hess;
			m_dummy = rhs.m_dummy;
			m_dummy_str = rhs.m_dummy_str;
			//m_dbl_dummy = rhs.m_dbl_dummy;
			//m_b_dummy = rhs.m_b_dummy;
			//m_str_dummy = rhs.m_str_dummy;
			m_2nd_order = rhs.m_2nd_order;

			m_magnetic = rhs.m_magnetic;
			m_thermodynamic = rhs.m_thermodynamic;
			m_physical = rhs.m_physical;
			m_kinetic = rhs.m_kinetic;
			m_phase_property = rhs.m_phase_property;
			m_misc_property = rhs.m_misc_property;

			m_st = make_shared<P_Statespace>(*rhs.m_st);

			m_y.clear();
			vector<shared_ptr<map <string, double>>>::const_iterator it;
            for (it = rhs.m_y.begin(); it!=rhs.m_y.end(); it++)
            {
                shared_ptr<map <string, double>> yy = make_shared<map <string, double>> ();
                *yy=**it;
                m_y.push_back(yy);
            }
			if (rhs.m_dGdy.size() != 0) {
				m_dGdy.clear();
				vector<shared_ptr<map <string, double>>>::const_iterator it;
				for (it = rhs.m_dGdy.begin(); it != rhs.m_dGdy.end(); it++)
				{
					shared_ptr<map <string, double>> eps = make_shared<map <string, double>>();
					*eps = **it;
					m_dGdy.push_back(eps);
				}
			}
			m_structure = rhs.m_structure;
		}
		return *this;
	}
public:
	string m_phase_name;
	int m_phase_id;
	int m_ph_nth;
	bool m_in_equilibrium;
	PAN_PHASE_STATUS m_status;
	double m_n_comp_per_formula;	// number of moles of components per formula (excluding vacancy)

	double m_mole;		// phase amount in moles, unit in moles
	double m_kg;		// phase amount in weight, unit in kg

	double m_f;		// phase molar fraction
	double m_fw;	// phase weight fraction

	bool m_f_fixed;	// phase molar fraction fixed
	int m_dummy;	// temporary variable
	string m_dummy_str;	// temporary string variable

	//double m_dbl_dummy;
	//bool m_b_dummy;
	//string m_str_dummy;
	pair<bool, P_2nd_order_Info> m_2nd_order;

	pair<bool, P_Property> m_magnetic;	// has magnetic property : Tc, BMag
	pair<bool, P_Property> m_physical;	// has physical properties : Vm, density, surface-tension, viscosity, conductivity
	pair<bool, P_Property_vector> m_kinetic;		// has kinetic properties
	pair<bool, P_Property_vector> m_thermodynamic;		// has m_thermodynamic_factor
	pair<bool, P_Property_vector> m_misc_property;		// has other properties, Qx, Qw
//	pair<bool, P_Property_vector> m_user_property;		// has user_property, user defined
	pair<bool, P_Property_vector> m_phase_property;		// has user_property or phase_property, user defined
	pair<bool, P_Property_vector> m_system_property;	// has system_property, user defined

public:
	set <string> m_comp;
	vector<shared_ptr<map <string, double>>> m_y;
	vector<shared_ptr<map <string, double>>> m_dGdy;
	vector<int> m_structure;

	shared_ptr<P_Statespace> m_st;
	double m_G, m_H, m_S, m_Cp;
	double m_G_ex, m_S_ex;
	double m_TPDF;

	pair <bool, Hessian> m_Hess;

public:
	double get_driving_force() {
		//return -m_TPDF;
		return -m_TPDF/(8.31451*m_st->m_T);
	}

	void print_debug(ostream& out) const
	{
		out << "-------------\n<Phase Point>\n";
		out << "Name=" << m_phase_name << " [" << m_phase_id << "] ";
		//if (m_dGdy.size() != 0) out << "(CSA) ";
		out << "m_status=" << PAN_PHASE_STATUS_NAME[m_status] << endl;
		out << "f" << ((m_f_fixed)? "$":"") << "=" << m_f << "  fw=" << m_fw << "  mole=" << m_mole << "  kg=" << m_kg;
		out << "  m_dummy=" << m_dummy;
		out << "  m_dummy_str=[" << m_dummy_str << "]\n";
		out << "[2nd_order:";
		if (m_2nd_order.first) out << "Yes] ""  dis_name=" << m_2nd_order.second.m_disorder_name << " [" << m_2nd_order.second.m_disorder_id << "]" << "  dy=" << m_2nd_order.second.m_dy << "  has_2nd_order=" << m_2nd_order.second.m_has_2nd_order;
		else out << "No] ";

		//if (m_2nd_order.first) {
		//	out << "  dis_name=" << m_2nd_order.second.m_disorder_name << " [" <<  m_2nd_order.second.m_disorder_id << "]" << "  dy=" <<  m_2nd_order.second.m_dy << "  has_2nd_order=" << m_2nd_order.second.m_has_2nd_order;
		//}
		out << endl;

		m_st->print_debug(out);

		if (m_y.size() > 0) {
			int id = 0;
			if (m_dGdy.size() == m_y.size()) {
				auto jt = m_dGdy.begin();
				for (auto it = m_y.begin(); it != m_y.end(); ++it, ++jt) {
					auto jt1 = (*jt)->begin();
					for (auto it1 = (*it)->begin(); it1 != (*it)->end(); it1++, jt1++)
					{
						out << "y[" << it1->first << ",#" << id << "]=" << it1->second;
						out << "  dGdy=" << jt1->second;
						out << endl;
					}
					id++;
				}
			}
			else {
				for (auto it = m_y.begin(); it != m_y.end(); ++it) {
					for (auto it1 = (*it)->begin(); it1 != (*it)->end(); it1++)
					{
						out << "y[" << it1->first << ",#" << id << "]=" << it1->second << endl;
					}
					id++;
				}
			}
			out << "Structure = [";
			for (auto i = m_structure.begin(); i != m_structure.end(); ++i) {
				if (i != m_structure.begin()) out << " ";
				out << *i;
			}
			out << "]" << endl;
		}
		
		int old_prec = (int)out.precision();
		out.precision(10);
		out << "G=" << m_G << " J/mole" << "  H=" << m_H << " J/mole" << "  S=" << m_S << " J/K.mole"
#if defined(_PEDEB) || defined(_PPDDEB)
			<< "  Cp=" << m_Cp << " J/K.mole" 
			<< "  G_ex=" << m_G_ex << " J/mole" << "  S_ex=" << m_S_ex << " J/K.mole" 			
#endif
			<< "  TPDF=" << m_TPDF << " J/mole" << "  HSN=" << m_Hess.second.m_HSN << endl;
		out.precision(old_prec);

		if (m_magnetic.first) {
			out << "  { Magnetic }\n";
			m_magnetic.second.print_debug(out);
		}

		if (m_thermodynamic.first) {
			out << "  { Thermodynamic }\n";
			out << "  { ignored 1 }\n";
//			m_thermodynamic.second.print_debug(out);
		}

		if (m_physical.first) {
			out << "  { Physical }\n";
			m_physical.second.print_debug(out);
		}

		if (m_kinetic.first) {
			out << "  { Kinetic }\n";
//			out << "  { ignored 2 }\n";
			m_kinetic.second.print_debug(out);
		}

		if (m_phase_property.first) {
			out << "  { Phase_Property }\n";
			m_phase_property.second.print_debug(out);
		}

		if (m_misc_property.first) {
			out << "  { Misc_Property }\n";
			m_misc_property.second.print_debug(out);
		}

		if (m_Hess.first) {
			m_Hess.second.print_debug(out);
		}
	}
};

enum EQUILIB_TYPE { eq_GLOBAL, eq_LOCAL, eq_INDIVIDUAL };
const char EQUILIB_TYPE_NAME[][max_num_chars] = {
	"eq_GLOBAL",
	"eq_LOCAL",
	"eq_INDIVIDUAL",
	"Type_not_defined"
};

class P_Point
{
public:
	P_Point(){
		m_stable=true;
		m_eq_type = eq_GLOBAL;
		m_dummy_str="";
		m_solvent="";
		m_reaction_equation="";
		m_driving_force=false;
		m_local_driving_force=false;
		m_thermodynamic.first=false;
		m_physical.first=false;
		m_kinetic.first=false;
		m_system_property.first=false;
		m_G = m_H = m_S = m_Cp = m_G_ex = m_S_ex = 0.0;
		m_mole = m_kg = m_vol = 1.0;
		m_fl = m_fs = 0;
		m_2nd_order = make_pair(false, -1);
	}

	virtual ~P_Point(){}
	P_Point(const P_Point& pt) {
		m_st = make_shared<P_Statespace>(pt.m_st);
		m_G = pt.m_G;
		m_H = pt.m_H;
		m_G_ex = pt.m_G_ex;
		m_S_ex = pt.m_S_ex;
		m_S = pt.m_S;
		m_Cp = pt.m_Cp;
		m_fl = pt.m_fl;
		m_fs = pt.m_fs;
		m_H_tot = pt.m_H_tot;
		m_H_Latent = pt.m_H_Latent;
		m_found = pt.m_found;
		m_stable = pt.m_stable;
		m_similar = pt.m_similar;
		m_pos = pt.m_pos;
		m_mole = pt.m_mole;
		m_kg = pt.m_kg;
		m_vol = pt.m_vol;
		m_dummy_str = pt.m_dummy_str;
		m_reaction_equation = pt.m_reaction_equation;
		m_driving_force = pt.m_driving_force;
		m_local_driving_force = pt.m_local_driving_force;
		m_eq_type = pt.m_eq_type;
		m_solvent = pt.m_solvent;
		m_thermodynamic = pt.m_thermodynamic;
		m_physical = pt.m_physical;
		m_kinetic = pt.m_kinetic;
		m_system_property = pt.m_system_property;
		m_contour = pt.m_contour;
		m_2nd_order = pt.m_2nd_order;

		for(auto it=pt.m_constraint.begin(); it!= pt.m_constraint.end(); it++)
		{
			m_constraint.push_back(*it);
		}

		for(auto it=pt.m_ppt.begin(); it!= pt.m_ppt.end(); it++)
		{
			shared_ptr<P_Phase_Point> ppp = make_shared<P_Phase_Point> (*it);
			m_ppt.push_back(ppp);
		}
		for(auto it=pt.m_ppt_entered.begin(); it!= pt.m_ppt_entered.end(); it++)
		{
			shared_ptr<P_Phase_Point> ppp = make_shared<P_Phase_Point> (*it);
			m_ppt_entered.push_back(ppp);
		}
		for(auto it=pt.m_ppt_dormant.begin(); it!= pt.m_ppt_dormant.end(); it++)
		{
			shared_ptr<P_Phase_Point> ppp = make_shared<P_Phase_Point> (*it);
			m_ppt_dormant.push_back(ppp);
		}
	}

	P_Point(shared_ptr<P_Point> pt_ptr) {
		m_st = make_shared<P_Statespace>(pt_ptr->m_st);
		m_G = pt_ptr->m_G;
		m_H = pt_ptr->m_H;
		m_G_ex = pt_ptr->m_G_ex;
		m_S_ex = pt_ptr->m_S_ex;
		m_S = pt_ptr->m_S;
		m_Cp = pt_ptr->m_Cp;
		m_fl = pt_ptr->m_fl;
		m_fs = pt_ptr->m_fs;
		m_H_tot = pt_ptr->m_H_tot;
		m_H_Latent = pt_ptr->m_H_Latent;
		m_found = pt_ptr->m_found;
		m_stable = pt_ptr->m_stable;
		m_similar = pt_ptr->m_similar;
		m_pos = pt_ptr->m_pos;
		m_mole = pt_ptr->m_mole;
		m_kg = pt_ptr->m_kg;
		m_vol = pt_ptr->m_vol;
		m_dummy_str = pt_ptr->m_dummy_str;
		m_reaction_equation = pt_ptr->m_reaction_equation;
		m_driving_force = pt_ptr->m_driving_force;
		m_local_driving_force = pt_ptr->m_local_driving_force;
		m_eq_type = pt_ptr->m_eq_type;
		m_solvent = pt_ptr->m_solvent;
		m_thermodynamic = pt_ptr->m_thermodynamic;
		m_physical = pt_ptr->m_physical;
		m_kinetic = pt_ptr->m_kinetic;
		m_system_property = pt_ptr->m_system_property;
		m_contour = pt_ptr->m_contour;
		m_2nd_order = pt_ptr->m_2nd_order;

		for(auto it=pt_ptr->m_constraint.begin(); it!= pt_ptr->m_constraint.end(); it++)
		{
			m_constraint.push_back(*it);
		}
		for(auto it=pt_ptr->m_ppt.begin(); it!= pt_ptr->m_ppt.end(); it++)
		{
			shared_ptr<P_Phase_Point> ppp = make_shared<P_Phase_Point> (*it);
			m_ppt.push_back(ppp);
		}
		for(auto it=pt_ptr->m_ppt_entered.begin(); it!= pt_ptr->m_ppt_entered.end(); it++)
		{
			shared_ptr<P_Phase_Point> ppp = make_shared<P_Phase_Point> (*it);
			m_ppt_entered.push_back(ppp);
		}
		for(auto it=pt_ptr->m_ppt_dormant.begin(); it!= pt_ptr->m_ppt_dormant.end(); it++)
		{
			shared_ptr<P_Phase_Point> ppp = make_shared<P_Phase_Point> (*it);
			m_ppt_dormant.push_back(ppp);
		}
}

	P_Point& operator=(const P_Point& rhs) {
		if (this != &rhs) {
			m_st = make_shared<P_Statespace>(rhs.m_st);
			m_G = rhs.m_G;
			m_H = rhs.m_H;
			m_G_ex = rhs.m_G_ex;
			m_S_ex = rhs.m_S_ex;
			m_S = rhs.m_S;
			m_Cp = rhs.m_Cp;
			m_fl = rhs.m_fl;
			m_fs = rhs.m_fs;
			m_H_tot = rhs.m_H_tot;
			m_H_Latent = rhs.m_H_Latent;
			m_found = rhs.m_found;
			m_stable = rhs.m_stable;
			m_similar = rhs.m_similar;
			m_pos = rhs.m_pos;
			m_mole = rhs.m_mole;
			m_kg = rhs.m_kg;
			m_vol = rhs.m_vol;
			m_dummy_str = rhs.m_dummy_str;
			m_reaction_equation = rhs.m_reaction_equation;
			m_driving_force = rhs.m_driving_force;
			m_local_driving_force = rhs.m_local_driving_force;
			m_eq_type = rhs.m_eq_type;
			m_solvent = rhs.m_solvent;
			m_thermodynamic = rhs.m_thermodynamic;
			m_physical = rhs.m_physical;
			m_kinetic = rhs.m_kinetic;
			m_system_property = rhs.m_system_property;
			m_contour = rhs.m_contour;
			m_phase_info = rhs.m_phase_info;
			m_2nd_order = rhs.m_2nd_order;

			m_constraint.clear();
			for(auto it=rhs.m_constraint.begin(); it!= rhs.m_constraint.end(); it++)
			{
				m_constraint.push_back(*it);
			}
			m_ppt.clear();
			for(auto it=rhs.m_ppt.begin();it!= rhs.m_ppt.end();it++)
            {
                shared_ptr<P_Phase_Point> ppp = make_shared<P_Phase_Point> (*it);
                m_ppt.push_back(ppp);
            }

			m_ppt_entered.clear();
			for(auto it=rhs.m_ppt_entered.begin();it!= rhs.m_ppt_entered.end();it++)
            {
                shared_ptr<P_Phase_Point> ppp = make_shared<P_Phase_Point> (*it);
                m_ppt_entered.push_back(ppp);
            }

			m_ppt_dormant.clear();
			for(auto it=rhs.m_ppt_dormant.begin();it!= rhs.m_ppt_dormant.end();it++)
            {
                shared_ptr<P_Phase_Point> ppp = make_shared<P_Phase_Point> (*it);
                m_ppt_dormant.push_back(ppp);
            }

		}
		return *this;
	}

public:
	shared_ptr<P_Coord> m_cd;
	shared_ptr<P_Statespace> m_st;
	vector <shared_ptr<P_Phase_Point>> m_ppt;
	vector <shared_ptr<P_Phase_Point>> m_ppt_entered;
	vector <shared_ptr<P_Phase_Point>> m_ppt_dormant;

	double m_G, m_H, m_S, m_Cp;
	double m_G_ex, m_S_ex;

	double m_mole;		// amount in moles, unit in moles
	double m_kg;		// amount in weight, unit in kg
	double m_vol;		// volume of point

	bool m_found;
	bool m_stable;
	bool m_similar;	// similar to  the previous point
	bool m_driving_force;	// calculate driving forces of entered and dormant phases
	bool m_local_driving_force;	// calculate local driving forces of entered or dormant phases in this point
	EQUILIB_TYPE m_eq_type;	// type: "global", "local", "individual"

	P_Position m_pos;	// position in corrdinate system, <u, v, w>
	P_Position m_dr;	// direction in corrdinate system, <du, dv, dw>
	double m_fl, m_fs, m_H_tot, m_H_Latent;	// for solidification
	string m_dummy_str;
	string m_reaction_equation;
	map<string, Phase_Info, icomp> m_phase_info;

	string m_solvent;
	pair<bool, P_Property_vector> m_thermodynamic;	// has m_thermodynamic properties
	pair<bool, P_Property> m_physical;	// has physical properties : Vm, surface-tension, viscosity, conductivity
	pair<bool, P_Property> m_kinetic;	// has kinetic properties
	pair<bool, P_Property> m_system_property;	// has system_property, user defined
	pair<string, double> m_contour;
	vector<pair<string, string>> m_constraint;	// such as T=1000, P=1e5

	pair<bool, int> m_2nd_order;	// second is for the ID of the ppt which is 2nd_order ppt
	// T K
	// P pascal
	// x(*) mole/mole
	// w(*) kg/kg
	// G J/mole
	// H J/mole
	// S J/(mole K)
	// Cp J/(mole K)
	// phase_name
	// invariant_reaction
	// mu(*) J/mole
	// f(@*) mole/mole
	// fw(@*) kg/kg
	// x(*@*) mole/mole
	// w(*@*) kg/kg
	// y(*@*) mole/mole
	// G(@*) J/mole
	// H(@*) J/mole
	// S(@*) J/(mole K)
	// Cp(@*) J/(mole K)
	// mu(@*) J/mole

	// fl mole/mole
	// fs mole/mole
	// H_tot J/mole
	// H_Latent J/mole

public:

	void set_T(double T) {
		m_st->m_T = T;
		for (auto i=m_ppt.begin(); i!=m_ppt.end(); ++i) {
			(*i)->m_st->m_T = T;
		}
	}

	void set_P(double P) {
		m_st->m_P = P;
		for (auto i=m_ppt.begin(); i!=m_ppt.end(); ++i) {
			(*i)->m_st->m_P = P;
		}
	}

	void set_T_P(double T, double P) {
		set_T(T);
		set_P(P);
	}

	int get_phase_id(string& name) {
		for (int i = 0; i < (int)m_ppt.size(); i++) {
			if (_stricmp((char*)m_ppt[i]->m_phase_name.c_str(), name.c_str()) == 0) {
				return i;	// return the first one even if more than one exist
				break;
			}
		}
		return -1;
	}

	double get_driving_force(string& phase_name)
	{
        auto it=m_ppt_dormant.begin();

        for(it = m_ppt_dormant.begin(); it!= m_ppt_dormant.end(); it++)
        {
            if (_stricmp((*it)->m_phase_name.c_str(), phase_name.c_str()) == 0)
            {
                break;
            }
        }

		if (it != m_ppt_dormant.end()) {
			return (*it)->get_driving_force();
		}

		return 0;
	}

	bool remove_zpf()
	{
		bool removed=false;

		while (1) {
			bool done = true;
			for(auto it = m_ppt.begin(); it!= m_ppt.end(); it++)
			{
				if ((*it)->m_f_fixed && (*it)->m_f == 0)
				{
					m_ppt.erase(it);
					removed = true;
					done = false;
					break;
				}
			}
			if (done) break;
		}

		return removed;
	}

	string set_T_fixed(bool b)
	{
		m_st->m_T_fixed = b;
		return "";
	}

	string set_P_fixed(bool b)
	{
		m_st->m_P_fixed = b;
		return "";
	}

	string set_T_P_fixed(bool bTP)
	{
		m_st->m_T_fixed = bTP;
		m_st->m_T_fixed = bTP;
		return "";
	}

	string set_T_P_fixed(bool bT, bool bP)
	{
		m_st->m_T_fixed = bT;
		m_st->m_T_fixed = bP;
		return "";
	}

	string set_f_fixed(bool b)
	{
		for(auto it = m_ppt.begin(); it!= m_ppt.end(); it++)
		{
			(*it)->m_f_fixed = b;
		}
		return "";
	}

	string set_f_fixed(string phase_name, bool b)
	{	// set the phase phase with name of "phase_name"
		for(auto it = m_ppt.begin(); it!= m_ppt.end(); it++)
		{
			if (phase_name == (*it)->m_phase_name) {
				(*it)->m_f_fixed = b;
				return "";
			}
		}
		return "Cannot find this phase!";
	}

	string set_f_fixed(int id, bool b)
	{
		if (id>=(int)m_ppt.size()) return "not found this phase";
		m_ppt[id]->m_f_fixed = b;
		return "";
	}

	void print_debug(ostream& out, int level = 0)
	{
		out << "\n<Point>  ";
//		if (m_2nd_order.first) out << "[2nd_order:" << m_2nd_order.second << "] ";
		out << "[2nd_order:";
		if (m_2nd_order.first) out << "Yes: " << m_2nd_order.second << "] ";
		else out  <<  "No] ";
		if (!m_dummy_str.empty()) out << "[" << m_dummy_str << "] ";
		out << ((m_found) ? "[found] ":"[not found] ");
		out << ((m_stable) ? "[stable] ":"[unstable] ");
		out << "eq_Type=" << EQUILIB_TYPE_NAME[m_eq_type] << " ";
#if defined(_PEDEB) || defined(_PPDDEB)
		out << ((m_similar) ? "[similar] ":"[different] ");
		out << ((m_driving_force) ? "[with_DF] ":"[no_DF] ");
		out << "u=" << m_pos.u << "  v=" << m_pos.v << "  w=" << m_pos.w << "  r=" << m_pos.r;
#endif
		out <<  "  fs=" << m_fs << "  fl=" << m_fl;
		if (!m_reaction_equation.empty()) out << " <" << m_reaction_equation << ">";
		out << endl;
		if (!m_contour.first.empty()) {
			out << "  contour <" << m_contour.first << ", " << m_contour.second << ">";
			out << endl;
		}
		if (m_constraint.size()>0) {
			for (auto i=m_constraint.begin(); i!=m_constraint.end(); ++i) {
				out << "  constraint <" << (*i).first << ", " << (*i).second << ">";
			}
			out << endl;
		}
		if (m_st) {
			m_st->print_debug(out);
		} else {
			out << "m_st not defined!\n";
		}
		out << "n=" << m_mole << " moles  G=" << m_G << " J/mole" 
			<< "  H=" << m_H << " J/mole" << "  S=" << m_S << " J/K.mole" << "  Cp=" << m_Cp  << " J/K.mole"
#if defined(_PEDEB) || defined(_PPDDEB)
			<< "  G_ex=" << m_G_ex << " J/mole" << "  S_ex=" << m_S_ex << " J/K.mole" 
			<< "  kg=" << m_kg << " kg" << " vol=" << m_vol
#endif
			<< endl;
		if (!m_solvent.empty()) {
	//		out << "  solvent="<< m_solvent <<endl;
		}

		if (m_thermodynamic.first) {
			out << "  { Thermodynamic }\n";
			out << "  { ignored 2 }\n";
			m_thermodynamic.second.print_debug(out);
		}

		if (m_physical.first) {
			out << "  { Physical }\n";
			m_physical.second.print_debug(out);
		}

		if (m_kinetic.first) {
			out << "  { Kinetic }\n";
			m_kinetic.second.print_debug(out);
		}

		if (m_system_property.first) {
			out << "  { system_Property }\n";
			m_system_property.second.print_debug(out);
		}

		if (m_constraint.size()>0) {
			out << "  Constraint :";
			for (auto i=m_constraint.begin(); i!=m_constraint.end(); ++i) {
				out << "  Constraint <" << (*i).first << ", " <<  (*i).second << ">";
			}
			out << endl;
		}

		out << "  @@ " << ((m_ppt.size() < 2)? "There is " : "There are ");
		out << m_ppt.size() << ((m_ppt.size() < 2)? " phase point" : " phase points") << " @@\n";

		for (auto it=m_ppt.begin();it!= m_ppt.end();it++)
        {
			(*it)->print_debug(out);
        }

		out << "  ####################################\n";
		out << "  ## " << ((m_ppt_entered.size() < 2)? "There is " : "There are ");
		out << m_ppt_entered.size() << ((m_ppt_entered.size() < 2)? " entered phase point" : " entered phase points") << " ##\n";
		if (level > 0 && m_ppt_entered.size()>0) {
			//out << " ----  Driving Force Points  ----\n";
			for (auto it=m_ppt_entered.begin();it!= m_ppt_entered.end();it++)
			{
				if (level >= 10) {
					(*it)->print_debug(out);
				} else {
				//	out << "  " << (*it)->m_phase_name << "  TPDF = " << (*it)->m_TPDF << " J/mole" << endl;
				}
			}
		}
		else {
			int cnt = 0;
			for (auto it = m_ppt_entered.begin(); it != m_ppt_entered.end(); it++)
			{
				out << "  " << (*it)->m_phase_name;
				if (++cnt > 10) {
					cnt = 0;
					out << endl;
				}
			}
			out << endl;
		}

		out << "  $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$\n";
		out << "  $$ " << ((m_ppt_dormant.size() < 2)? "There is " : "There are ");
		out << m_ppt_dormant.size() << ((m_ppt_dormant.size() < 2)? " dormant phase point" : " dormant phase points") << " $$\n";
		if (level > 0 && m_ppt_dormant.size()>0) {
			//out << " ----  Driving Force Points  ----\n";
			for (auto it=m_ppt_dormant.begin();it!= m_ppt_dormant.end();it++)
			{
				if (level >= 10) {
					(*it)->print_debug(out);
					//out << "  " << (*it)->m_phase_name << "  TPDF = " << (*it)->m_TPDF << " J/mole" << endl;
				}
				else {
					out << "  " << (*it)->m_phase_name << "  TPDF = " << (*it)->m_TPDF << " J/mole" << endl;
				}
			}
		}
		else {
			int cnt = 0;
			for (auto it = m_ppt_dormant.begin(); it != m_ppt_dormant.end(); it++)
			{
				out << "  " << (*it)->m_phase_name;
				if (++cnt > 10) {
					cnt = 0;
					out << endl;
				}
			}
			out << endl;
		}

		if (!m_phase_info.empty()) {
			out << "[Phase Info]\n";
			for (auto ph=m_phase_info.begin(); ph!=m_phase_info.end(); ++ph) {
				if ((*ph).second.residual_liq)
				{
					out << (*ph).first << " {residual}" << ":\n";
				}
				else
				{
					out << (*ph).first << ":\n";;
				}
				(*ph).second.print_debug(out);
			}
		}
		out << "\n";
	}

	void print_debug_short(ostream& out, int typ=0) const
	{
//		out << "\n<<Point>>\n";
//		out << "<Point>  ";
		out << m_st->m_T << " K  ";
		out << "logP=" << log10(m_st->m_P) << "  ";
		out << "[" << m_ppt.size() << "]  ";	
		
		out << "{";
		for (auto it=m_ppt.begin();it!= m_ppt.end();it++)
        {
			if (it!=m_ppt.begin()) out << " ";
			out << (*it)->m_phase_name;
			out << ((*it)->m_f_fixed?"$":"");
			out << " " << (*it)->m_f;

			if ((*it)->m_structure.size()>0) {
				out << "[";
				for (auto i = (*it)->m_structure.begin(); i != (*it)->m_structure.end(); ++i) {
					if (i != (*it)->m_structure.begin()) out << " ";
					out << *i;
				}
				out << "]";
			}

        }
		out << "}  ";
#ifndef PANENGINE
		out << "u(" << m_pos.u << ") v(" << m_pos.v << ") w(" << m_pos.w << ") " << " r(" << m_pos.r << ") " << "vol(" << m_vol << ") ";
#endif
		if (!m_contour.first.empty()) out << "  contour <" << m_contour.first << ", " << m_contour.second << ">";
		if (m_constraint.size()>0) {
			for (auto i=m_constraint.begin(); i!=m_constraint.end(); ++i) {
				out << "  constraint <" << (*i).first << ", " << (*i).second << ">";
			}
		}
		if (!m_reaction_equation.empty()) out << " <" << m_reaction_equation << "> ";



		for (auto it=m_st->m_comp.begin();it!=m_st->m_comp.end();it++)
        {
			out << "x[" << (*it).first << "]=" << (*it).second->m_x << "  ";
        }

		if (m_st->m_species.size()>0) {
			for (auto it=m_st->m_species.begin();it!=m_st->m_species.end();it++)
			{
				out << "sp[" << (*it).second->m_var.m_name << "]=" << (*it).second->m_var.m_value << "  ";
			}
		}

		if (typ!=0) out << endl;
		for (auto it=m_ppt.begin();it!= m_ppt.end();it++)
        {
			if (typ==0 && it!=m_ppt.begin()) out << "+";
			out << " " << (*it)->m_phase_name;
#ifdef PANENGINE
			out << "[" << (*it)->m_f << "]";
#else
			out << "[" << "id(" << (*it)->m_phase_id << ")," <<  "nth(" << (*it)->m_ph_nth << ")," << ((*it)->m_f_fixed?"$":"") << (*it)->m_f;
			
			out << ",ThF(" << (*it)->m_thermodynamic.first << ")";
			out << "]";
#endif		
			if (typ!=0) out << endl;
		}

		out << " G=" << m_G;
		out << " H=" << m_H;
		out << " S=" << m_S;
		out << " Cp=" << m_Cp;
		out << endl;
		if (m_ppt_dormant.size()>0) {
			out << "  DF: ";

			for(auto itt = m_ppt_dormant.begin(); itt!= m_ppt_dormant.end(); itt++)
			{
				out << " " << (*itt)->m_phase_name << "(" << (*itt)->get_driving_force() << ")";
			};

			out << endl;
		}
		if (m_thermodynamic.first) {
			m_thermodynamic.second.print_debug(out);
		}
	}

};

class P_Line
{
public:
	P_Line() {
		m_type = ""; m_contour.first = ""; m_contour.second = ""; m_id = 0;	}
	P_Line(string typ) { m_type = typ; m_contour.first = ""; m_contour.second = "";  m_id = 0;}
	P_Line(pair<string, string> contour) { m_type = ""; m_contour=contour;  m_id = 0;}
	P_Line(string typ, pair<string, string> contour) { m_type = typ; m_contour=contour;  m_id = 0;}

	virtual ~P_Line(){;}
	P_Line(shared_ptr<P_Line> line_ptr) {
		for (auto it=line_ptr->m_point.begin();it!= line_ptr->m_point.end();it++)
		{
			shared_ptr<P_Point> ppt = make_shared<P_Point> (*it);
			m_point.push_back(ppt);
		}
	}

public:
	vector <shared_ptr<P_Point>> m_point;
	string m_type;
	pair<string, string> m_contour;
	int m_id;
	string m_dummy;
	P_Coord m_cd;

public:
	void print_debug(ostream& out, int level=0) const
	{
		out << "\n<<Line " << m_id << ">>\n";
		if (m_point.size() > 1) {
			out << "[";
			for (auto i = m_point[1]->m_ppt.begin(); i != m_point[1]->m_ppt.end(); ++i) {
				if (i != m_point[1]->m_ppt.begin()) out << " + ";
				out << (*i)->m_phase_name;
			}
			out << "]\n";
		}
		out << "m_type=" << m_type << "  ";
		out << "m_contour=(" << m_contour.first << ", " << m_contour.second << ")  ";
		out << "dummy=(" << m_dummy << ")  ";
		out << endl;
		out << "Line has " << m_point.size() << " points.\n";
		int id=0;
		for (auto it = m_point.begin();it!= m_point.end();it++, ++id)
		{
			bool print_short_form = false;
			if (print_short_form) {
				out << id << " ";
				if (id<10) out << " ";
			//	(*it)->print_debug(out);
				(*it)->print_debug_short(out);
			} else {
				(*it)->print_debug(out, level);
			}
		}
	}

	// print the first b and the last e points
	// if b<0 && e<0, print all points
	void print_debug_short(ostream& out, int b = -1, int e = -1) const
	{
		out << "\n<<Line " << m_id << ">>\n";
		if (m_point.size() > 1) {
			out << "[";
			for (auto i = m_point[1]->m_ppt.begin(); i != m_point[1]->m_ppt.end(); ++i) {
				if (i != m_point[1]->m_ppt.begin()) out << " + ";
				out << (*i)->m_phase_name;
			}
			out << "]\n";
		}
		out << "m_type=" << m_type << "  ";
		out << "m_contour=(" << m_contour.first << ", " << m_contour.second << ")  ";
		out << "dummy=(" << m_dummy << ")  ";
		out << endl;

		out << "Line has " << m_point.size() << " points.\n";
		int id=0;
		int sz=m_point.size();

		if ((b<0 && e<0) || b>=sz || e>=sz || b+e>=sz) {
			for (auto it = m_point.begin(); it!= m_point.end(); it++, ++id)
			{
				out << id << " ";
				if (id<10) out << " ";
				(*it)->print_debug_short(out);
			}
		} else {
			if (b>=0) {
				id=0;
				for (auto it = m_point.begin(); it!= m_point.end(); it++, ++id)
				{
					out << id << " ";
					if (id<10) out << " ";
					(*it)->print_debug_short(out);
					if (id > b) break;
				}
			}

			if (e>=0) {
				id = sz - e;
				if (id < 0) id = 0;
				else out << "... ...\n";
				for (auto it = m_point.begin()+id; it!= m_point.end(); it++, ++id)
				{
					out << id << " ";
					if (id<10) out << " ";
					(*it)->print_debug_short(out);
				}
				return;
			}
		}
	}
};
