/////////////////////////////////////
//                                 //
//         PanEngine API           //
//                                 //
//         Version 2020            //
//                                 //
//        CompuTherm, LLC          //
//      Copyright, 2012-2020       //
//                                 //
//       www.computherm.com        //
//                                 //
/////////////////////////////////////

//=====================================
//  Main Program for Testing PanEngine
//=====================================

#include "std.h"
#include  <sstream>
#ifdef WIN32
#include <windows.h>
#endif

#define OUTPUT_TO_FILE

extern int PanEngineX_Test_1();
extern int PanEngineX_Test_2();
extern int PanEngineX_Test_3();
extern int PanEngineX_Test_4();
extern int PanEngineX_Test_5();
extern int PanEngineX_Test_6();
extern int PanEngineX_Test_7();
extern int PanEngineX_Test_8();
extern int PanEngineX_Test_9();

long long counter()
{
#ifdef WIN32
	LARGE_INTEGER li;
	QueryPerformanceCounter(&li);

	return li.QuadPart;
#else
	return 0;
#endif
}

long long frequency()
{
#ifdef WIN32
	LARGE_INTEGER li;
	QueryPerformanceFrequency(&li);

	return li.QuadPart;
#else
	return 0;
#endif
}

string format_time(const long long start, const long long finish)
{
#ifndef WIN32
	return "";
#endif

	ostringstream oss;

	if ((finish - start) * 1000 < frequency())
	{
		oss << (finish - start) * 1000000.0 / frequency() << " microseconds";
	}
	else if (finish - start < frequency())
	{
		oss << (finish - start) * 1000.0 / frequency() << " milliseconds";
	}
	else
	{
		oss << (finish - start) * 1.0 / frequency() << " seconds";
	}

	return oss.str();
}

int test_PanEngineX(int example_ID) {
	long long start_t;
	long long end_t;

	start_t = counter();

#ifdef OUTPUT_TO_FILE
	std::streambuf* cout_sbuf = std::cout.rdbuf(); // save original sbuf 
	ostringstream oss;
	oss << "test_" << example_ID << ".dat";
	std::ofstream out(oss.str());
	std::cout.rdbuf(out.rdbuf()); // redirect 'cout' to a 'out'
#endif
	switch (example_ID) {
	case 1:
		// Point Equilibrium
		PanEngineX_Test_1();
		break;
	case 2:
		// Global Equilibria for Multiple Points
		PanEngineX_Test_2();
		break;
	case 3:
		// Local Equilibria for Multiple Points
		PanEngineX_Test_3();
		break;
	case 4:
		// Solidification Simulation
		PanEngineX_Test_4();
		break;
	case 5:
		// Solidification Simulation for Multiple Alloys
		PanEngineX_Test_5();
		break;
	case 6:
		// Liquidus Surface and Slopes
		PanEngineX_Test_6();
		break;
	case 7:
		// Lowest Liquidus Point
		PanEngineX_Test_7();
		break;
	case 8:
		// Kinetic Properties and Hessian Matrix
		PanEngineX_Test_8();
		break;
	case 9:
		// Parallel Tangent Equilibrium
		PanEngineX_Test_9();
		break;
	}
#ifdef OUTPUT_TO_FILE
	std::cout.rdbuf(cout_sbuf); // restore the original stream buffer 
#endif

	end_t = counter();
	cout << " (" << format_time(start_t, end_t) << ")" << endl;

	return 1;
}

int test_PanEngineX_API(int example_ID, pair<int, int> example_id_range)
{		
	if (example_ID == 0) {
		return 0;
	} else
	if (example_ID >= 1 && example_ID <= example_id_range.second) {
		cout << "Example " << example_ID << " ... " << endl;
		test_PanEngineX(example_ID);
	} else if (example_ID == -1) {
		for (int i = 1; i <= example_id_range.second; ++i) {
			cout << "Example " << i << " ...  ";
			test_PanEngineX(i);
		}
	}

	return 1;
}

void show_menu(pair<int, int>& id_range) {
	cout << "=================================\n";
	cout << "|| PanEngine API Test Examples ||\n";
	cout << "||        Version 2020         ||\n";
	cout << "||       CompuTherm, LLC       ||\n";
	cout << "=================================\n";
	cout << "\n\n";
	cout << "Select An Example:\n";
	cout << "   1 : Point Equilibrium\n";
	cout << "   2 : Equilibria for Multiple Points\n";
	cout << "   3 : Local Equilibria for Multiple Points\n";
	cout << "   4 : Solidification Simulation\n";
	cout << "   5 : Solidification Simulation for Multiple Alloys\n";
	cout << "   6 : Liquidus Surface and Slopes\n";
	cout << "   7 : Lowest Liquidus Point\n";
	cout << "   8 : Kinectic Properties and Hessian Matrix\n";
	cout << "   9 : Parallel Tangent Equilibrium\n";
	cout << "   0 : Exit\n";
	cout << "  -1 : Test All Examples\n";

	id_range.first = -1;
	id_range.second = 9;
}

int main() 
{
	pair<int, int> example_id_range;
	show_menu(example_id_range);

	int example_ID = 1;
	cin >> example_ID;
	cout << endl;
	if (example_ID < example_id_range.first || example_ID > example_id_range.second) 
	{
		return 1;
	}
	cout << endl;

	test_PanEngineX_API(example_ID, example_id_range);
	cout << "\nCalculation is done.\n\n";

//#ifndef OUTPUT_TO_FILE
	system("pause");
//#endif

	return 1;
}